/*=========================================================================*/
/*                          LockFile.c                                     */
/*                                                                         */
/*  Locks all files in the Desktop file system                             */
/*                                                                         */
/*                                                                         */
/*                                                                         */
/*               IconLock Version 1.0 Copyright (c) 1999                   */
/*                         Ziff Davis, Inc.							       */
/*                       All rights reserved.							   */
/*        First appeared in PC Magazine, US Edition, August, 1999	       */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/

#include "common.h"
#include "LockDll.h"
#include "Shlobj.h"

int iFileCount;
static int iLockCount;
static HGLOBAL hLock;
void LockDesktopFile( LPCSTR pFile );
void ParseDirectories( LPSTR pPath );


//=====================================================================
//  FUNCTION:		LockFiles
//
//  PURPOSE:		Gets the directories that comprise the desktop and
//					locks them.
//  PARAMETERS:		
//
//  RETURN VALUE:	
//
//  COMMENTS:		
//=====================================================================
void LockFiles( )
{
LPITEMIDLIST pidl;   
LPMALLOC pMalloc;
char sz[MAX_PATH],szCommon[MAX_PATH];

	iFileCount = 0;

	if( SUCCEEDED(SHGetMalloc( &pMalloc )) ) 
	{
		SHGetSpecialFolderLocation( NULL, CSIDL_DESKTOP, &pidl );
		SHGetPathFromIDList( pidl, sz );
		
		if( *sz ) {
			ParseDirectories( sz );
		}

		SHGetSpecialFolderLocation( NULL, CSIDL_DESKTOPDIRECTORY, &pidl );
		SHGetPathFromIDList( pidl, szCommon );

		if( *szCommon ) {
			if( strcmp( szCommon, sz ) )
			{			
				ParseDirectories( szCommon );
			}
		}

		SHGetSpecialFolderLocation( NULL, CSIDL_COMMON_DESKTOPDIRECTORY, &pidl );
		SHGetPathFromIDList( pidl, sz );
	
		if( *sz ) {
			ParseDirectories( sz );
		}

		pMalloc->lpVtbl->Free( pMalloc,pidl );
		pMalloc->lpVtbl->Release( pMalloc );
	}

}


//=====================================================================
//  FUNCTION:		ParseDirectories
//
//  PURPOSE:		Recursive function that enumerates all folders and 
//					subfolders in the Desktop.
//
//  PARAMETERS:		Directory path
//
//  RETURN VALUE:	
//
//  COMMENTS:		
//=====================================================================
void ParseDirectories( LPSTR pPath )
{
WIN32_FIND_DATA finddata;
HANDLE hFile;
char szFile[MAX_PATH];
char szDir[MAX_PATH];
DWORD dwAttr;
	
	
	//since this is a recursive function, check the number of locked files
	//for an excessive amount.  we don't want to hang the system if the user
	//has a corrupted file system.
	if( ++iFileCount > 9999 )
		return;
	
	wsprintf( szDir, "%s\\*.*", pPath  );	

	hFile = FindFirstFile( szDir, &finddata );

	if( hFile != INVALID_HANDLE_VALUE )
	{
		while( TRUE )
		{	
			if( ( strcmp(finddata.cFileName, "." ) !=0 ) && ( strcmp(finddata.cFileName, ".." )!= 0 ) )
			{		
				wsprintf( szFile, "%s\\%s", pPath, finddata.cFileName );

				dwAttr = GetFileAttributes( szFile );
				if( (dwAttr & FILE_ATTRIBUTE_DIRECTORY) && (dwAttr != -1 ) )
					ParseDirectories( szFile );
				else 
					LockDesktopFile( szFile );
			}

			if( !FindNextFile( hFile, &finddata ) )
				break;
		}

		FindClose( hFile );
	}
}

//=====================================================================
//  FUNCTION:	LockDesktopFile
//
//  PURPOSE:	Opens the file with	shared read and write access mode
//
//  PARAMETERS:	Fully qualified filename.	
//
//  RETURN VALUE:	
//
//  COMMENTS:		
//=====================================================================
void LockDesktopFile( LPCSTR pFile )
{
DWORD dwAccess;
DWORD dwSize = (iLockCount+1) * sizeof( HGLOBAL );
HGLOBAL* pLock;

	//this allows an app to open or execute the file
	//however prevents renames and deletes
	if( bWinNT )
		dwAccess = GENERIC_READ | GENERIC_WRITE;
	else
		dwAccess = 0;	

	if( !hLock )
		hLock = GlobalAlloc( GHND, dwSize );
	else
		hLock = GlobalReAlloc( hLock, dwSize, GMEM_ZEROINIT );

	if( hLock )
	{
		pLock = GlobalLock( hLock );	
		pLock[iLockCount++] = CreateFile( pFile, dwAccess,
							  FILE_SHARE_READ | FILE_SHARE_WRITE , NULL, OPEN_EXISTING, 
							  FILE_ATTRIBUTE_NORMAL, NULL );

		GlobalUnlock( hLock );
	}

}


//=====================================================================
//  FUNCTION:		UnlockDesktopFiles( )
//
//  PURPOSE:		Closes the handle to locked desktop files	
//
//  PARAMETERS:		
//
//  RETURN VALUE:	
//
//  COMMENTS:		
//=====================================================================
void UnlockDesktopFiles( )
{
	int i;
	HGLOBAL* pLock;

	if( hLock )
	{
		pLock = GlobalLock( hLock );	
		for( i = 0; i < iLockCount; i++ )
			CloseHandle( pLock[i] );

		GlobalUnlock( hLock );
		GlobalFree( hLock );
		hLock = 0;
	}

}

