/*=========================================================================*/
/*                              LOCKUTIL.C                                 */
/*                                                                         */
/*         Various utility functions used in LockDLL                       */
/*                                                                         */
/*                                                                         */
/*                                                                         */
/*               IconLock Version 1.0 Copyright (c) 1999                   */
/*                         Ziff Davis, Inc.							       */
/*                       All rights reserved.							   */
/*        First appeared in PC Magazine, US Edition, August, 1999	       */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/
#include "common.h"
#include "lockdll.h"
//#define _WIN32_IE 0x0400	//~	1.1
#include <shlwapi.h>




//=====================================================================
//  FUNCTION:	IsWinNT		          
//
//  PURPOSE:		                                
//
//  PARAMETERS:		                
//
//  RETURN VALUE:	TRUE if WinNT	      
//
//  COMMENTS:		                          
//=====================================================================
BOOL IsWinNT( )
{
	OSVERSIONINFO  osvi; 
	osvi.dwOSVersionInfoSize = sizeof(osvi); 
	GetVersionEx(&osvi); 
 
	return( VER_PLATFORM_WIN32_NT == osvi.dwPlatformId );
}
 

//=====================================================================
//  FUNCTION:	GetShell32Version		          
//
//  PURPOSE:	Returns version of the shell		                                
//
//  PARAMETERS:		                
//
//  RETURN VALUE:	      
//
//  COMMENTS:		                          
//=====================================================================
BOOL GetShell32Version( LPDWORD pdwMajor, LPDWORD pdwMinor, BOOL bUseShell32 )
{
HINSTANCE   hShell32;

	if( IsBadWritePtr(pdwMajor, sizeof(DWORD)) || IsBadWritePtr(pdwMinor, sizeof(DWORD)))
			return FALSE;

	*pdwMajor = 0;
	*pdwMinor = 0;
	
	//note: Under WinNT shdocvw.dll and shell32.dll do not sync the version numbers
	//with IE5.  Shell32.dll still is 4.72 while shdocvw.dll is 5.00. 
    //Therefore we need to examine shdocvw.dll under WinNT to get the real version number
    //of the Shell under IE5. 
	if( bUseShell32 )
        hShell32 = LoadLibrary(TEXT( "shell32.dll" ) );
    else
        hShell32 = LoadLibrary(TEXT( "shdocvw.dll" ) );

	if( hShell32 )
   {
		HRESULT           hr = S_OK;
		DLLGETVERSIONPROC pDllGetVersion;
   
	   /*
	   You must get this function explicitly because earlier versions of the DLL 
	   don't implement this function. That makes the lack of implementation of the 
	   function a version marker in itself.
	   */
		pDllGetVersion = (DLLGETVERSIONPROC)GetProcAddress( hShell32, TEXT("DllGetVersion") );
   
		if(pDllGetVersion)
		{
			DLLVERSIONINFO    dvi;
      
			ZeroMemory(&dvi, sizeof(dvi));
			dvi.cbSize = sizeof(dvi);
		   
			hr = (*pDllGetVersion)(&dvi);
      
			if(SUCCEEDED(hr))
			{
				*pdwMajor = dvi.dwMajorVersion;
				*pdwMinor = dvi.dwMinorVersion;
         }
      }
   else
      {
		  /*
		  If GetProcAddress failed, the DLL is a version previous to the one 
		  shipped with IE 3.x.
		  */
		  *pdwMajor = 4;
		  *pdwMinor = 0;
      }
   
   FreeLibrary( hShell32 );

   return TRUE;
   }

return FALSE;
}

//=====================================================================
//  FUNCTION:		IsActiveDesktop( )
//
//  PURPOSE:		checks the registry for the active desktop value
//					see Q216840 from MSDN online
//  PARAMETERS:			                
//
//  RETURN VALUE:	true if windows desktop update is installed
//
//  COMMENTS:		added in v1.1                          
//=====================================================================
BOOL IsActiveDesktop( )
{
HKEY hKey;
DWORD dwSize;
BYTE  szValue[MAX_PATH];
#define ACTIVE_DESKTOP	"SOFTWARE\\Classes\\CLSID\\{AEB6717E-7E19-11d0-97EE-00C04FD91972}\\InProcserver32"
 
	if( RegOpenKeyEx( HKEY_LOCAL_MACHINE, ACTIVE_DESKTOP, 0, KEY_ALL_ACCESS, &hKey ) == ERROR_SUCCESS )
	{
		dwSize = sizeof( szValue );
		RegQueryValueEx( hKey,"", 0, NULL, szValue, &dwSize );
		RegCloseKey( hKey );
		szValue[sizeof(szValue)-1] = 0;
		strlwr( (LPSTR)szValue );
		if( strstr( (LPSTR)szValue, "shell32.dll" ) )
			return( TRUE );
	}

	return( FALSE );

}



//=====================================================================
//  FUNCTION:		GetHelpFile		          
//
//  PURPOSE:		Locates the full path to our help file		                                
//
//  PARAMETERS:			                
//
//  RETURN VALUE:	full path and entry window for the help file      
//
//  COMMENTS:		                          
//=====================================================================
LPSTR GetHelpFile( LPSTR pHelpPath )
{
char szName[MAX_PATH];
LPSTR p;

	//Qualify the INI file to the same location as our DLL
	GetModuleFileName( GetModuleHandle( SZ_DLLNAME ), szName, sizeof( szName ) );
	p = strrchr( szName, '\\' );

	if( p )
		*(++p) = 0;
	else 
		p = szName;

	strcat( p, SZ_HELPFILE );
	strcpy( pHelpPath, szName );
	strcat( pHelpPath, ">proc1" );

	return( pHelpPath );

}

//=====================================================================
//  FUNCTION:		ShowError
//
//  PURPOSE:		Displays a MessageBox using a formatted string 
//                  resource.
//
//  PARAMETERS:		wID -- the resource ID
//
//  RETURN VALUE:	none
//
//  COMMENTS:       
//=====================================================================
void _cdecl ShowError( WORD wID,... )
{
char szMsg[512];
char szBuf[512];
va_list argptr;
LPCSTR p;

	if( LoadString( hInstGlobal, wID, szMsg, sizeof( szMsg ) ) )
		p = szMsg;
	else
		p = "Unknown Error.";

	va_start( argptr, wID );
 	wvsprintf( szBuf, p, argptr );
    va_end( argptr);

	MessageBox( GetDesktopWindow(), szBuf, SZ_APPNAME, MB_OK | MB_ICONERROR | MB_SETFOREGROUND );

}


//=====================================================================
//  FUNCTION:		DEBUGOUT
//
//  PURPOSE:		Sends a formatted string to the debug window
//                  Only used for testing in DEBUG 
//
//  PARAMETERS:	    format string and parameters	
//
//  RETURN VALUE:	none
//
//  COMMENTS:       Ex.  DEBUGOUT( "hMenu = %x", hMenu );       
//=====================================================================
void _cdecl DebugOut_CompiledOutOfReleaseBuild( char *format, ... )
{
#ifdef _DEBUG
    char szBuf[512];
	char s[22];
    va_list argptr;

    va_start( argptr, format );
	wvsprintf( szBuf, format, argptr );
    va_end( argptr);

	wsprintf( s, "%X   ",GetCurrentThreadId() );
	OutputDebugString( s );
	OutputDebugString( szBuf );
	OutputDebugString( "\n\r" );
#endif
}

