/*=========================================================================*/
/*                              LOCKDATA.C                                 */
/*                                                                         */
/*            Functions for reading writing the ICONLOCK.INI               */
/*            and ICONLOCK.BIN files.                                      */
/*                                                                         */
/*                                                                         */
/*                                                                         */
/*               IconLock Version 1.0 Copyright (c) 1999                   */
/*                         Ziff Davis, Inc.							       */
/*                       All rights reserved.							   */
/*        First appeared in PC Magazine, US Edition, August, 1999	       */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/

#include "common.h"
#include "LockDLL.h"

void GetDataFilePath( LPSTR pFileName );
void LockDataFiles( BOOL bLock, LPSTR pDataFile );


//=====================================================================
//  FUNCTION:		MyWritePrivateProfileString
//
//  PURPOSE:		Locks and unlocks the ini file before writing to it
//
//  PARAMETERS:     	    
//
//  RETURN VALUE:	
//
//  COMMENTS:
//=====================================================================
BOOL MyWritePrivateProfileString( LPCSTR pBuf, LPCSTR lpKey, LPCSTR lpString, LPCSTR lpFileName )
{
BOOL bRet;	

	LockDataFiles( FALSE, NULL );
	bRet = WritePrivateProfileString( pBuf, lpKey, lpString, lpFileName );
	LockDataFiles( TRUE, NULL );
	return( bRet );
}


//=====================================================================
//  FUNCTION:		MyGetPrivateProfileString
//
//  PURPOSE:		Locks and unlocks the ini file before reading it
//
//  PARAMETERS:     	    
//
//  RETURN VALUE:	
//
//  COMMENTS:
//=====================================================================
BOOL MyGetPrivateProfileString( LPCSTR lpAppName, LPCSTR lpKey, LPCSTR lpDefault,LPSTR lpString, DWORD dwSize, LPCSTR lpFileName )
{
BOOL bRet;	

	LockDataFiles( FALSE, NULL );
	bRet = GetPrivateProfileString( lpAppName, lpKey, lpDefault, lpString, dwSize, lpFileName );
	LockDataFiles( TRUE, NULL );
	return( bRet );
}


//=====================================================================
//  FUNCTION:		LockDataFiles
//
//  PURPOSE:		Locks or unlocks the ICONLOCK.INI and ICONLOCK.BIN
//					files.
//  PARAMETERS:     	    
//
//  RETURN VALUE:	
//
//  COMMENTS:
//=====================================================================
void LockDataFiles( BOOL bLock, LPSTR pDataFile )
{
static HANDLE hDataFile, hIniFile;

	if( bLock && iconlock.byDesktopLocked && iconlock.byLockDataFiles )
	{
		if( !hDataFile && pDataFile )
		{
			hDataFile = CreateFile( pDataFile, GENERIC_WRITE | GENERIC_READ ,
						0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
			if( hDataFile != INVALID_HANDLE_VALUE )
				LockFile( hDataFile, 0, 0, -1, 0 );
			else
				hDataFile = 0;
		}

		if( !hIniFile )
		{
			hIniFile = CreateFile( szIniFile, GENERIC_WRITE | GENERIC_READ ,
						0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
			if( hIniFile != INVALID_HANDLE_VALUE )
				LockFile( hIniFile, 0, 0, -1, 0 );
			else
				hIniFile = 0;
		}

	} else {	//unlock
		if( hDataFile && pDataFile )	
		{
			UnlockFile( hDataFile, 0,0,-1,0 );
			CloseHandle( hDataFile ); 
			hDataFile = 0;
		}

		if( hIniFile ) {
			UnlockFile( hIniFile, 0,0,-1,0 );
			CloseHandle( hIniFile );
			hIniFile = 0;
		}
		
	
	}
	
}


//=====================================================================
//  FUNCTION:		UnLockDataFiles
//
//  PURPOSE:		called when the desktop is unlocked
//					
//  PARAMETERS:     	    
//
//  RETURN VALUE:	
//
//  COMMENTS:
//=====================================================================
void UnLockDataFiles( )
{
	char szFile[MAX_PATH];

	GetDataFilePath( szFile );
	LockDataFiles( FALSE, szFile );
}



//=====================================================================
//  FUNCTION:		Encrypt
//
//  PURPOSE:		Performs a simple encryption of the password
//
//  PARAMETERS:	    string to string to encrypt, pointer to data
//                  structure
//
//  RETURN VALUE:	none
//
//  COMMENTS:       This encryption could be more robust however it
//                  should be good enough for casual implementations.
//
//=====================================================================
void Encrypt( LPCSTR pString, PENCRYPT pData )
{
	int i,j;

	pData->byEncryptKey = (BYTE)GetTickCount( ); //random xor value
	pData->byStringLen = min( strlen( pString ), MAX_PASSWORD_LEN );
	
	for( i = 0; i < pData->byStringLen; i++ )
		pData->byString[i] =  pString[i] ^ ( pData->byEncryptKey + ( i * pData->byEncryptKey ) );

	//fill the rest with junk
	for( j = 0 ; i < MAX_PASSWORD_LEN; i++,j++ )
		pData->byString[i] = pData->byString[j] ^ i;
}


//=====================================================================
//  FUNCTION:		Decrypt
//
//  PURPOSE:		Converts an encrypted string back to normal text
//
//  PARAMETERS:     the encrypted string and data structure to fill	    
//
//  RETURN VALUE:	
//
//  COMMENTS:
//=====================================================================
void Decrypt( LPSTR pString, PENCRYPT pData )
{
int i;
	    		
	for( i = 0; i < pData->byStringLen; i++  )
		*pString++ =  pData->byString[i] ^ ( pData->byEncryptKey + ( i * pData->byEncryptKey ) );
	
	*pString = 0;

}



//=====================================================================
//  FUNCTION:		GetDataFilePath
//
//  PURPOSE:		fully qualifies the file name using the location
//					of the DLL process
//  PARAMETERS:     filename
//
//  RETURN VALUE:	qualified name -- copied back to passed string
//
//  COMMENTS:		pFileName must be MAX_PATH
//=====================================================================
void GetDataFilePath( LPSTR pFileName )
{
char szFullPath[MAX_PATH];
LPSTR p;

	GetModuleFileName( GetModuleHandle( SZ_DLLNAME ), szFullPath, sizeof( szFullPath ) );
	p = strrchr( szFullPath, '\\' );

	if( p )
		*(++p) = 0;
	else 
		p = szFullPath;

	strcat( p, SZ_DATAFILE );
	strcpy( pFileName, szFullPath );
}


//=====================================================================
//  FUNCTION:		SaveDataFile
//
//  PURPOSE:		saves the iconlock struct to ICONLOCK.BIN
//					
//  PARAMETERS:     
//
//  RETURN VALUE:	
//
//  COMMENTS:		
//=====================================================================
void SaveDataFile( PICONLOCK_DATA pData, PMENUSTRINGS hMenuStrings )
{
char szFile[MAX_PATH];
HANDLE hFile;
int i;
DWORD dwWritten;

	GetDataFilePath( szFile );
	LockDataFiles( FALSE, szFile );
	
    hFile = CreateFile(szFile, GENERIC_WRITE,
                       0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL );
   
    //set the version number
	iconlock.byVersionNumber = DATAFILE_VERSION;
	
	if( hFile != INVALID_HANDLE_VALUE )
	{
		WriteFile( hFile, (LPCVOID)pData, sizeof( ICONLOCK_DATA ), &dwWritten,	NULL );
		
		for( i = 0; i < iconlock.iNumberOfMenuStrings; i++ )
			WriteFile( hFile, (LPCVOID)hMenuStrings[i].szMenuString, MAX_MENUSTRING_LEN, &dwWritten, NULL );

		CloseHandle( hFile );

	} else {
		ShowSystemError( "Error opening ICONLOCK.BIN file!" );
	}

	LockDataFiles( TRUE, szFile );
	
}





//=====================================================================
//  FUNCTION:		ReadsDataFile
//
//  PURPOSE:		reads the ICONLOCK.BIN file into the global
//                  iconlock structure
//					
//  PARAMETERS:     see comments
//
//  RETURN VALUE:	
//
//  COMMENTS:		if pFile is not null then it is being called from 
//                  the uninstall function.  The dll may not be loaded
//                  so we need to locate the ICONLOCK.BIN file first.
//=====================================================================
void ReadDataFile( LPSTR pFile )
{
char szFile[MAX_PATH];
HANDLE hFile;
int i,iCount;
DWORD dwRead;
PMENUSTRINGS pMenuStrings; 

	ZeroMemory( &iconlock, sizeof( iconlock ) );
	
	if( pFile == 0 ) {
		GetDataFilePath( szFile );
		pFile = szFile;
	}
	LockDataFiles( FALSE, pFile );

    hFile = CreateFile( pFile, GENERIC_READ,
                       0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
   
    if( hFile != INVALID_HANDLE_VALUE )
	{
		ReadFile( hFile, (LPVOID)&iconlock, sizeof( ICONLOCK_DATA ), &dwRead, NULL );

		if( hMenuStrings )
			GlobalFree( hMenuStrings );
		
		if( iconlock.iNumberOfMenuStrings )
		{
			//set a minimum value for the menu strings in case the bin file
			//was somehow corrupted.			
			iconlock.iNumberOfMenuStrings = min( 256, iconlock.iNumberOfMenuStrings );
			iCount =  iconlock.iNumberOfMenuStrings;
			hMenuStrings = GlobalAlloc( GPTR, iCount * MAX_MENUSTRING_LEN );
			pMenuStrings = (PMENUSTRINGS)hMenuStrings;
			iconlock.iNumberOfMenuStrings = 0;

			for( i = 0; hMenuStrings && i < iCount; i++ )
			{
				ReadFile( hFile, pMenuStrings[i].szMenuString, MAX_MENUSTRING_LEN, &dwRead,	NULL );
				if( dwRead == 0 )
					break;
				iconlock.iNumberOfMenuStrings++;
				
			}

		}
		CloseHandle( hFile );
		
		//Check the version number -- perform adjustments here
		if( iconlock.byVersionNumber !=  DATAFILE_VERSION )
		{
			ShowError( IDS_INVALID_VERSION_ERROR, SZ_DATAFILE );
			ZeroMemory( &iconlock, sizeof( iconlock ) );
		}

		
		//check to see that the locked flag is set to '1'
		//if the file was somehow corrupted, the force locked to off
		if( iconlock.byDesktopLocked != 1 )
			iconlock.byDesktopLocked = 0;
		
		if( iconlock.byExplorerLocked != 1 )
			iconlock.byExplorerLocked = 0;

	} else {
		ShowSystemError( "Error reading ICONLOCK.BIN file!" );
	}
	
	LockDataFiles( TRUE, pFile );
}


//=====================================================================
//  FUNCTION:		ShowSystemError
//
//  PURPOSE:		shows message box using FormatMessage()
//                  iconlock structure
//  PARAMETERS:     
//
//  RETURN VALUE:	
//
//  COMMENTS:		
//=====================================================================
void ShowSystemError( LPSTR pError )
{
   LPVOID lpMsgBuf;
   char szErr[1024];
   DWORD rc;

   rc = FormatMessage(	FORMAT_MESSAGE_ALLOCATE_BUFFER |
                        FORMAT_MESSAGE_FROM_SYSTEM |     
                        FORMAT_MESSAGE_IGNORE_INSERTS,    
						NULL, GetLastError(),    
						MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
						(LPSTR)&lpMsgBuf, 0, NULL);


	wsprintf( szErr, "%s\n\n%s",pError, lpMsgBuf );

	MessageBox( GetFocus( ), szErr, SZ_APPNAME, MB_OK | MB_ICONHAND | MB_SETFOREGROUND );

    LocalFree( lpMsgBuf );
} 


