/*=========================================================================*/
/*                              LOCKDLG.C                                  */
/*                                                                         */
/*                 Dialog Box functions for LOCKDLL.DLL                    */
/*                                                                         */
/*                                                                         */
/*               IconLock Version 1.0 Copyright (c) 1999                   */
/*                         Ziff Davis, Inc.							       */
/*                       All rights reserved.							   */
/*        First appeared in PC Magazine, US Edition, August, 1999	       */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/
#include "common.h"
#include "lockdll.h"
#include "..\help\cntxmap.h"


typedef struct {
	char szResolution[30];
	BOOL bDisable;
} LISTBOX_DATA, *PLISTBOX_DATA;


HFONT CreateURLFont( HWND );


//=====================================================================
//  FUNCTION:		RemoveTabs
//
//  PURPOSE:	
//              
//  PARAMETERS:	    
//
//  RETURN VALUE:	
//
//  COMMENTS:       
//=====================================================================
void RemoveTabs( LPSTR pString )
{

	while( pString && *pString )
	{
		if( *pString == '\t' )
		{
			*pString = 0;
			break;
		}

		pString++;
	}

}


//=====================================================================
//  FUNCTION:		RestoreProc
//
//  PURPOSE:		Allows the user to restore a custom icon layout
//                  description.  Also added the option to delete a
//                  layout.
//
//  PARAMETERS:	    standard
//
//  RETURN VALUE:	standard
//
//  COMMENTS:       
//=====================================================================
BOOL EXPORTED CALLBACK RestoreProc( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
char szBuf[80];

	switch( uMsg )
	{
		case WM_INITDIALOG:
		{
			FillDialogControl( GetDlgItem( hDlg, IDD_LISTBOX ), LB_ADDSTRING, TRUE);
			
			if( SendDlgItemMessage( hDlg, IDD_LISTBOX, LB_GETCOUNT, 0, 0 ) == 0 )
			{
				ShowError( IDS_SAVEDNAMES );
				EndDialog( hDlg, TRUE );
			} else {
                SendDlgItemMessage( hDlg, IDD_LISTBOX, LB_SETCURSEL, 0 ,0 );
            }
			break;
		}
			
		case WM_DELETEITEM:
			if ((UINT)((LPDELETEITEMSTRUCT)lParam)->itemData) 
				GlobalFree((HGLOBAL)(((LPDELETEITEMSTRUCT)lParam)->itemData) ); 
			break;

		case WM_DISPLAYCHANGE:
			SendDlgItemMessage( hDlg, IDD_LISTBOX, LB_RESETCONTENT, 0, 0 );
			FillDialogControl( GetDlgItem( hDlg, IDD_LISTBOX ), LB_ADDSTRING, TRUE);
			break;

		case WM_DRAWITEM:
		{
			char szBuf[300];
			LPDRAWITEMSTRUCT lpDIS = (DRAWITEMSTRUCT FAR *) lParam;  // for convenience.
			HDC hDC = lpDIS->hDC;
			DWORD itemData = lpDIS->itemData;
			COLORREF cvText; 
			COLORREF cvBack; 
			BOOL bDisable = FALSE;
			HBRUSH hbr;
			TEXTMETRIC tm;
			HGLOBAL hData;
			PLISTBOX_DATA pData;

            //  Only process messages for our listbox.
            if( lpDIS->CtlID != IDD_LISTBOX )
                return FALSE;
    
            if( lpDIS->itemID == -1 ) // empty listbox 
            {
                return TRUE;
            }

            //get the text for the combo box
            SendDlgItemMessage( hDlg, IDD_LISTBOX, LB_GETTEXT, lpDIS->itemID, (LPARAM)(LPCSTR)szBuf );

			//check to see if the saved resolution is enabled
			hData = (HGLOBAL)SendDlgItemMessage(hDlg, IDD_LISTBOX, LB_GETITEMDATA, lpDIS->itemID, 0 );		
			if( hData )
			{
				pData = (PLISTBOX_DATA)GlobalLock( hData );
				bDisable = pData->bDisable;
				SetDlgItemText( hDlg, IDD_RESOLUTION, pData->szResolution );
				GlobalUnlock( hData ); 
			} else {
				SetDlgItemText( hDlg, IDD_RESOLUTION, "" );
			}

			
			if( bDisable )
			{
				cvText =  GetSysColor(COLOR_GRAYTEXT);
 
				cvBack = GetSysColor((lpDIS->itemState & ODS_SELECTED) 
									? COLOR_HIGHLIGHT : COLOR_WINDOW);  
			} else {
				cvText =  GetSysColor((lpDIS->itemState & ODS_SELECTED) 
									? COLOR_HIGHLIGHTTEXT : COLOR_WINDOWTEXT);  
 
				cvBack = GetSysColor((lpDIS->itemState & ODS_SELECTED) 
									? COLOR_HIGHLIGHT : COLOR_WINDOW);  
			}
						
            hbr = CreateSolidBrush( cvBack ); 
			FillRect( hDC, &(lpDIS->rcItem), hbr);   
			
			SetTextColor(lpDIS->hDC,  cvText );
			
   			SetBkMode( hDC, TRANSPARENT );
			GetTextMetrics( hDC, &tm );
			

			if( tm.tmAveCharWidth == 0 )
				tm.tmAveCharWidth = 8;
			
			DrawText( hDC, szBuf, -1, &(lpDIS->rcItem), DT_LEFT | DT_NOPREFIX );
			if (lpDIS->itemState & ODS_FOCUS) 
			{ 
			//	DrawFocusRect(lpDIS->hDC, &lpDIS->rcItem);  
			} 

			DeleteObject( hbr );
			EnableWindow( GetDlgItem( hDlg, IDOK ), bDisable == FALSE );
            return( TRUE );
        }


		case WM_COMMAND:
			switch( LOWORD( wParam ) )
			{
				case IDD_LISTBOX:
					if( HIWORD(wParam) != LBN_DBLCLK )
						break;
					//fall through				
				case IDOK:
				{
					DWORD dwItem = SendDlgItemMessage( hDlg, IDD_LISTBOX, LB_GETCURSEL, 0 ,0 );
					if( dwItem != LB_ERR )
					{
						SendDlgItemMessage( hDlg, IDD_LISTBOX, LB_GETTEXT, dwItem, (LPARAM)(LPSTR)szBuf );	
						RestoreIconLocations( szBuf );
					}
					EndDialog( hDlg, TRUE );
				}

				case IDCANCEL:
					EndDialog( hDlg, FALSE );
					break;
				
				case IDD_DELETE:
				{
					char szMsg[256];
					DWORD dwItem = SendDlgItemMessage( hDlg, IDD_LISTBOX, LB_GETCURSEL, 0 ,0 );
					if( dwItem != LB_ERR  )
					{
						SendDlgItemMessage( hDlg, IDD_LISTBOX, LB_GETTEXT, dwItem, (LPARAM)(LPSTR)szBuf );	
						
						wsprintf( szMsg, "Are you sure you want to delete %s?", szBuf );
                        
						if( MessageBox( NULL, szMsg, SZ_APPNAME, MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND ) == IDYES )
						{
							MyWritePrivateProfileString( szBuf, NULL, NULL, szIniFile );
							SendDlgItemMessage( hDlg, IDD_LISTBOX, LB_RESETCONTENT,0 ,0  );
							FillDialogControl( GetDlgItem( hDlg, IDD_LISTBOX ), LB_ADDSTRING, TRUE );
						}			
					}
					break;
				}
									
				case IDD_HELP:
				{
					char szHelp[MAX_PATH];
					WinHelp( hDlg, GetHelpFile( szHelp ), HELP_CONTEXT, IDH_restore );
				}
				break;
	
				default:
					return( FALSE );
			}

			return( TRUE );
	}	
	return( FALSE );
}



//=====================================================================
//  FUNCTION:		InsertDimensions
//
//  PURPOSE:		
//
//  PARAMETERS:	    
//
//  RETURN VALUE:	
//
//  COMMENTS:       
//=====================================================================
void InsertDimensions( HWND hCtrl, WORD wCommand, LPCSTR pKey )
{
	char szKey[80],szBuf[2048];
	int iIndex;
	PLISTBOX_DATA pData;
	HGLOBAL hData;

	strcpy( szKey, pKey );

	MyGetPrivateProfileString( szKey, SZ_DIMENSIONKEY, "", szBuf, sizeof( szBuf ), szIniFile );	

	iIndex = SendMessage( hCtrl, wCommand, 0, (LPARAM)(LPSTR)szKey );

	wsprintf( szKey, "%ux%u",GetSystemMetrics( SM_CXSCREEN ), 
  							 GetSystemMetrics( SM_CYSCREEN ) ); 

	hData = GlobalAlloc( GHND, sizeof( LISTBOX_DATA ) );
	if( hData )
	{
		pData = (PLISTBOX_DATA)GlobalLock( hData );
		strncpy( pData->szResolution, szBuf, sizeof( pData->szResolution) );
		pData->bDisable = strcmp(szKey,szBuf);
		GlobalUnlock( hData ); 
		SendMessage( hCtrl, LB_SETITEMDATA, iIndex, (LPARAM)hData );
	}
	
}

//=====================================================================
//  FUNCTION:		FillDialogControl
//
//  PURPOSE:		Fills a listbox or combobox control with 
//                  section names from our INI file.
//
//  PARAMETERS:	    hCtrl -- handle to the dialog control
//                  wCommand -- either LB_ADDSTRING or CB_ADDSTRING
//
//  RETURN VALUE:	none
//
//  COMMENTS:       
//=====================================================================
void FillDialogControl( HWND hCtrl, WORD wCommand, BOOL bAddResolution )
{
	char szBuf[2048];
	int iLength;
	LPSTR pStart,pEnd;
	
	iLength = MyGetPrivateProfileString( NULL, NULL, "", szBuf, sizeof( szBuf ), szIniFile );
	pStart = pEnd = szBuf;

	if( iLength ) {
		while( *pEnd ) {        //terminate at null,null
			while( *pEnd++ );    //find end of string
			
			//If the section name contains the string 'IconLock'
            //then it is one of our private sections. Don't add it to the list.
            if( strstr( pStart, SZ_APPNAME ) == 0 )
			{
				if( bAddResolution)
					InsertDimensions( hCtrl, wCommand, pStart );
				else
					SendMessage( hCtrl, wCommand, 0, (LPARAM)(LPSTR)pStart );
			}

			pStart = pEnd;
		}
	}

}

//=====================================================================
//  FUNCTION:		AboutProc
//
//  PURPOSE:		Displays the About box and Version info
//
//  PARAMETERS:	    standard
//
//  RETURN VALUE:	standard
//
//  COMMENTS:       
//=====================================================================
BOOL EXPORTED CALLBACK AboutProc( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static HFONT hURLFont;

	switch( uMsg )
	{
		case WM_INITDIALOG:
			InitAboutBox( hDlg );
			hURLFont = CreateURLFont( hDlg );
			SetClassLong( GetDlgItem( hDlg, IDD_URL ), GCL_HCURSOR, (LONG)LoadCursor( hInstGlobal, MAKEINTRESOURCE( IDC_HAND ) ) );
			break;
		
		case WM_CTLCOLORSTATIC: 
		{
			HDC hdc = (HDC) wParam;
			HWND hWnd = (HWND) lParam; // handle of static control 

			if( GetDlgCtrlID( hWnd ) == IDD_URL ) 
			{
				SelectObject( hdc, hURLFont );
				SetTextColor( hdc, RGB(0,0,255 ) );
				SetBkMode( hdc, TRANSPARENT );
				return((LRESULT) GetStockObject( NULL_BRUSH ) );
			}
			return (FALSE);

		}
		break;

		case WM_COMMAND:
			switch( LOWORD( wParam ) )
			{

				case IDD_URL:
				{
					char szURL[512];
					if( LoadString( hInstGlobal, IDS_URL, szURL, sizeof( szURL ) ) )
						ShellExecute(NULL, NULL, szURL, NULL, NULL, SW_SHOW);
				}
				//fall through	

				case IDOK:
					if( hURLFont )
						DeleteObject( hURLFont );
					hURLFont = 0;
					
					EndDialog( hDlg, TRUE );
					break;
				
				
				default:
					return( FALSE );
			}

			return( TRUE );
	}
	return( FALSE );
}


//=====================================================================
//  FUNCTION:		InitAboutBox
//
//  PURPOSE:		Retrieves the Version info
//
//  PARAMETERS:	    handle to dialog
//
//  RETURN VALUE:	none
//
//  COMMENTS:       
//=====================================================================
void InitAboutBox( HWND hDlg )
{
    char  szFullPath[256];
    DWORD dwVerHnd;
    DWORD dwVerInfoSize;
 
    // Get version information from the application
    GetModuleFileName( hInstGlobal, szFullPath, sizeof(szFullPath) );
    dwVerInfoSize = GetFileVersionInfoSize( szFullPath, &dwVerHnd );

	if( dwVerInfoSize )
    {
        // If we were able to get the information, process it:
        HANDLE  hMem;
        LPVOID  lpvMem;
        char    szGetName[256];
        int     cchRoot;
        int     i;

        hMem = GlobalAlloc(GMEM_MOVEABLE, dwVerInfoSize);
        lpvMem = GlobalLock(hMem);
        GetFileVersionInfo(szFullPath, dwVerHnd, dwVerInfoSize, lpvMem);
        lstrcpy(szGetName, "\\StringFileInfo\\040904b0\\");
        cchRoot = lstrlen(szGetName);

        // Walk through the dialog items that we want to replace:
        for( i = 0; i < 2; i++ )
        {
            BOOL  fRet;
            UINT  cchVer = 0;
            LPSTR lszVer = NULL;
            char  szResult[256];
			WORD  wID;

			if( i == 0 )
				wID = IDD_COPYRIGHT;
			else
				wID = IDD_VERSION;


            GetDlgItemText( hDlg, wID, szResult, sizeof(szResult));
            lstrcpy( &szGetName[cchRoot], szResult );
            fRet = VerQueryValue( lpvMem, szGetName, &lszVer, &cchVer );

            if( fRet && cchVer && lszVer )
            {
                // Replace dialog item text with version info
                lstrcpy( szResult, lszVer );
                SetDlgItemText( hDlg, wID, szResult );
            }
        }
        GlobalUnlock( hMem );
        GlobalFree( hMem );
    }
    return;
}


//=====================================================================
//  FUNCTION:		EnterPassWordProc
//
//  PURPOSE:		Prompts the user for the password to lock, unlock,
//                  or change password
//
//  PARAMETERS:	    standard
//
//  RETURN VALUE:	TRUE if the password was entered correctly
//
//  COMMENTS:       If no password has been defined we just exit        
//=====================================================================
BOOL EXPORTED CALLBACK EnterPassWordProc( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
char szBuf[256];


	switch( uMsg )
	{
		case WM_INITDIALOG:
		{
			HICON hIcon;

			Decrypt( szBuf, &iconlock.Prompt );
			if( szBuf[0] == 0 )
				lstrcpy( szBuf, "Enter password:" );
			SetDlgItemText( hDlg, IDD_QUESTION, szBuf );
			SetDlgItemText( hDlg, IDD_INCORRECT, "" );

            SendDlgItemMessage( hDlg, IDD_PASSWORD, EM_LIMITTEXT, MAX_PASSWORD_LEN, 0 );
				
			if( iconlock.byDesktopLocked )
				hIcon = LoadIcon( hInstGlobal, MAKEINTRESOURCE( IDI_LOCKED ) );
			else
				hIcon = LoadIcon( hInstGlobal, MAKEINTRESOURCE( IDI_UNLOCKED ) );
			SendDlgItemMessage( hDlg, IDC_LOCKICON, STM_SETICON, (WPARAM)hIcon, 0 );
			DestroyIcon( hIcon );

			//exit now if no password has been entered
			Decrypt( szBuf, &iconlock.Password );
			if( szBuf[0] == 0 )
				EndDialog( hDlg, TRUE );

		}
			break;

		case WM_COMMAND:
			switch( LOWORD( wParam ) )
			{
				char szPassword[256];
				HICON hIcon;
				
				case IDOK:
					GetDlgItemText( hDlg, IDD_PASSWORD, szBuf, sizeof( szBuf ) );
					Decrypt( szPassword, &iconlock.Password );
					 
					if( strcmpi( szBuf, szPassword ) && strlen( szPassword ) )
					{
						SetDlgItemText( hDlg, IDD_INCORRECT, "Incorrect, please try again." );
						SetDlgItemText( hDlg, IDD_PASSWORD, "" );
						SetFocus( GetDlgItem( hDlg, IDD_PASSWORD ) ); 
						MessageBeep( MB_ICONEXCLAMATION );
						break;
					}
					if( !iconlock.byDesktopLocked )
						hIcon = LoadIcon( hInstGlobal, MAKEINTRESOURCE( IDI_LOCKED ) );
					else
						hIcon = LoadIcon( hInstGlobal, MAKEINTRESOURCE( IDI_UNLOCKED ) );
					
					SendDlgItemMessage( hDlg, IDC_LOCKICON, STM_SETICON, (WPARAM)hIcon, 0 );
					UpdateWindow( hDlg );
		
					Sleep( 100 );
					DestroyIcon( hIcon );
					EndDialog( hDlg, TRUE );
					break;

				case IDCANCEL:
					EndDialog( hDlg, FALSE );
					break;
				
				default:
					return( FALSE );
			}

			return( TRUE );
	}
	return( FALSE );
}



//=====================================================================
//  FUNCTION:		SaveAsProc
//
//  PURPOSE:		Allows the user to save an desktop layout with a 
//                  user defined name.
//
//  PARAMETERS:	    standard
//
//  RETURN VALUE:	none
//
//  COMMENTS:  
//=====================================================================
BOOL EXPORTED CALLBACK SaveAsProc( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
char szBuf[256];

	switch( uMsg )
	{
		case WM_INITDIALOG:
			FillDialogControl( GetDlgItem( hDlg, IDD_SAVEAS ), CB_ADDSTRING, FALSE );
			break;

		case WM_COMMAND:
			switch( LOWORD( wParam ) )
			{
				case IDOK:
					GetDlgItemText( hDlg, IDD_SAVEAS, szBuf, sizeof( szBuf ) );
					if( strlen( szBuf ) )
					{
						SaveIconLocations( szBuf );
						EndDialog( hDlg, TRUE );
					}
					break;

				case IDCANCEL:
					EndDialog( hDlg, FALSE );
					break;

				case IDD_HELP:
				{
					char szHelp[MAX_PATH];
					WinHelp( hDlg, GetHelpFile( szHelp ), HELP_CONTEXT, IDH_save_as );
				}
				break;

				
				default:
					return( FALSE );
			}

			return( TRUE );

	}
	return( FALSE );

}


//=====================================================================
//  FUNCTION:		CreateURLFont
//
//  PURPOSE:		creates an underlined font for the about box
//
//  PARAMETERS:     
//
//  RETURN VALUE:	
//
//  COMMENTS:		
//=====================================================================
HFONT CreateURLFont( HWND hDlg ) 
{
	
	TEXTMETRIC tm;
	LOGFONT    lf;
	HDC hdc;
	char szFaceName[LF_FACESIZE];
	
	
	hdc = GetDC( hDlg );
	SelectObject( hdc, GetStockObject( DEFAULT_GUI_FONT ) );
	GetTextMetrics( hdc, &tm );
	GetTextFace( hdc, sizeof( szFaceName ), szFaceName );
	ReleaseDC( hDlg, hdc );

	ZeroMemory( &lf, sizeof( LOGFONT ) );	

	lf.lfHeight			 = tm.tmHeight;
	lf.lfWidth			 = tm.tmAveCharWidth;
	lf.lfWeight			 = FW_THIN;
	lf.lfUnderline       = 1;
	lf.lfPitchAndFamily  = tm.tmPitchAndFamily;
	lf.lfCharSet         = tm.tmCharSet;
	lf.lfOutPrecision    = OUT_DEFAULT_PRECIS;
	lf.lfQuality         = DEFAULT_QUALITY;
 	strcpy( lf.lfFaceName, szFaceName );
	return( CreateFontIndirect( &lf ) );
	
}


