/*=========================================================================*/
/*                             LockDLL.H                                   */
/*                                                                         */
/*               Header file for LockDLL.c and LockDlg.c                   */
/*                                                                         */
/*                                                                         */
/*=========================================================================*/

//this allows us to put all	variables in one header
#ifndef MAIN
	#define EXTERN extern
#else
	#define EXTERN
#endif


#define MAX_PASSWORD_LEN 50
#define MAX_MENUSTRING_LEN 30
#define DATAFILE_VERSION 1	

typedef struct 
{
	BYTE byEncryptKey;
	BYTE byString[MAX_PASSWORD_LEN];
	BYTE byStringLen;
} ENCRYPT, *PENCRYPT;


typedef struct
{
	char szMenuString[MAX_MENUSTRING_LEN];
} MENUSTRINGS, *PMENUSTRINGS;


//Structure layout for the ICONLOCK.BIN file
//data after the iNumberOfMenuStrings is variable
//Needs a CRC checksum!
typedef struct 
{
	BYTE	byVersionNumber;		//for future updates, set to DATAFILE_VERSION
	ENCRYPT Password;
	ENCRYPT Prompt;
	BYTE	byDesktopLocked;
	BYTE	byExplorerLocked;
	BYTE    byTaskBarLocked;
	BYTE	byAutoRestore;
	BYTE    byLockDataFiles;
	BYTE	byLockFilesFolders;
	BYTE	byLockCommonDlgs;
	BYTE	byOptionsDlgShown;		//for first time run ( display options dialog )
	int		iNumberOfMenuStrings;	//must be last in list -- strings follow this
} ICONLOCK_DATA, *PICONLOCK_DATA;

/****************************/
/*         Globals          */
/****************************/
EXTERN HINSTANCE hInstGlobal;		//DLL instance handle
EXTERN HWND  hListView;				//global for ProgMans listview 
EXTERN HMENU hIconLockMenu;			//handle to our popup menu
EXTERN char  szIniFile[MAX_PATH];	//path to iconlock.ini
EXTERN HGLOBAL hMenuStrings;		//global handle to the menu items to be disabled
EXTERN ICONLOCK_DATA iconlock;		//our iconlock.bin file
EXTERN BOOL bWinNT;					//flag for NT 
EXTERN BOOL bActiveDesktop;			//flag for IE4 shell versions 4.7 and up

#ifndef MAIN
	EXTERN const char SZ_UNDO[];
	EXTERN const char SZ_DLLNAME[];
	EXTERN const char SZ_FILE_INI[];
	EXTERN const char SZ_DATAFILE[];
	EXTERN const char SZ_HELPFILE[];
	EXTERN const char SZ_UNLOCK[];  
	EXTERN const char SZ_DIMENSIONKEY[]; 
#else
	const char SZ_DLLNAME[]     = { "LockDLL" };
	const char SZ_FILE_INI[]    = { "ICONLOCK.INI" };
	const char SZ_DATAFILE[]    = { "ICONLOCK.BIN" };
	const char SZ_UNDO[]        = { "IconLock Undo" };
	const char SZ_HELPFILE[]    = { "IconLock.hlp" };
	const char SZ_UNLOCK[]      = { "&Unlock Desktop" }; 
	const char SZ_DIMENSIONKEY[]= { "<Dimensions>" }; 
#endif



/****************************/
/*         Prototypes       */
/****************************/
LRESULT EXPORTED CALLBACK CallWndProc( int nCode, WPARAM wParam, LPARAM lParam );
LRESULT EXPORTED CALLBACK KeyboardProc( int code, WPARAM wParam, LPARAM lParam );
BOOL EXPORTED CALLBACK EditPassWordProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
BOOL EXPORTED CALLBACK EnterPassWordProc( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam );
BOOL EXPORTED CALLBACK SaveAsProc( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam );
BOOL EXPORTED CALLBACK RestoreProc( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam );
BOOL EXPORTED CALLBACK AboutProc( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam );
BOOL EXPORTED CALLBACK MenuOptionsProc( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam );
LRESULT EXPORTED CALLBACK TaskBarProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
LRESULT EXPORTED CALLBACK ListViewProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );

BOOL CALLBACK EnumProc( HWND, LPARAM );
void FillDialogControl( HWND hCtrl, WORD wCommand, BOOL );
void GetScreenDimensions( LPSTR pSectionName );
void DisableRestoreMenuOption( HMENU hMenu );
void InitAboutBox( HWND hDlg );
void RestoreIconLocations( LPCSTR pSection );
void SaveIconLocations( LPCSTR pSection );
void LoadOurMenu( );
BOOL IsAutoArrangeEnabled( );
void GetListViewHandle( );
void _cdecl ShowError( WORD wID,... );
void Encrypt( LPCSTR pString, PENCRYPT pData  );
void Decrypt( LPSTR pString, PENCRYPT pData  );
void SaveDataFile( );
void ReadDataFile( LPSTR );
void ShowSystemError( LPSTR pError );
int  ShowOptionsDlg( HWND hwndOwner );
BOOL EXPORTED CALLBACK OptionsProc( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam );
HMENU CopyMenu( HMENU hDesktopMenu );
void DisableDesktopMenuItems( HMENU hMenu );
void DisableIconLockMenuItems( HMENU hMenu );
void SetLockMenuOptions( HMENU hMenu );
BOOL HasUndoBeenSaved( );
BOOL HasResolutionBeenSaved( );
void HookTaskBar( BOOL );
LPSTR GetHelpFile( LPSTR pHelpPath );
void UnLockDataFiles( );
BOOL MyWritePrivateProfileString( LPCSTR pBuf, LPCSTR lpKey, LPCSTR lpString, LPCSTR lpFileName );
BOOL MyGetPrivateProfileString( LPCSTR lpAppName, LPCSTR lpKey, LPCSTR lpDefault,LPSTR lpString, DWORD dwSize, LPCSTR lpFileName );
BOOL IsWinNT( );
BOOL GetShell32Version( LPDWORD pdwMajor, LPDWORD pdwMinor, BOOL );
void RemoveDesktopINI( );
UINT ShowShutdownDLG( );
void SignalIconLock( );
LONG RecursiveRegDeleteKey( HKEY hKey, LPCSTR lpKey );
void LockFiles( );
void UnlockDesktopFiles( );
BOOL IsActiveDesktop( );

//debug strings are not complied into the objects if _DEBUG is not defined
void _cdecl DebugOut_CompiledOutOfReleaseBuild( char *format,... );
#ifdef _DEBUG
	#define DEBUGOUT DebugOut_CompiledOutOfReleaseBuild
#else
	#define DEBUGOUT 1 ? (void)0 : DebugOut_CompiledOutOfReleaseBuild
#endif 
