/*=========================================================================*/
/*                              LOCKPROP.C                                 */
/*                                                                         */
/*                 Dialog Box functions for LOCKDLL.DLL                    */
/*                 Password, Menu Options, Lock Options                    */
/*                                                                         */
/*                                                                         */
/*               IconLock Version 1.0 Copyright (c) 1999                   */
/*                         Ziff Davis, Inc.							       */
/*                       All rights reserved.							   */
/*        First appeared in PC Magazine, US Edition, August, 1999	       */
/*                                                                         */
/*                                                                         */
/*=========================================================================*/
#include "common.h"
#include "lockdll.h"
#include "..\help\cntxmap.h"

void EnableRemoveButtons( HWND hDlg );
void FillDefaultsList( HWND );
void CenterDialog( HWND );
void AddDefaults( HWND );
void CALLBACK PropSheetCallback(HWND hwndPropSheet, UINT uMsg, LPARAM lParam);
void SetAddEditState( HWND hDLg, BOOL );
void CheckAllMenuDefaults( HWND hDlg );

BOOL bApplyChanges;	//return code from property sheet
HWND hWndProp; //1.1

//=====================================================================
//  FUNCTION:		ShowOptionsDlg
//
//  PURPOSE:		Display property sheet dialog
//
//  PARAMETERS:		owner of the property sheet dialog
//
//  RETURN VALUE:	n/a
//
//  COMMENTS:		Because this dialog is running in the 
//                  process space of ProgMan, the user won't
//                  be able to right or left click any icons on the 
//                  desktop while the dialog is running -- even when 
//                  modeless.
//=====================================================================
int ShowOptionsDlg( HWND hwndOwner )
{
	MSG msg;
	PROPSHEETPAGE psp[3];
    PROPSHEETHEADER psh;
	
	bApplyChanges = FALSE;

	InitCommonControls( );
	
	ZeroMemory( &psp, sizeof( psp ) );
	ZeroMemory( &psh, sizeof( psh ) );

    psp[0].dwSize = sizeof(PROPSHEETPAGE);
    psp[0].dwFlags = PSP_USETITLE | PSP_HASHELP;
    psp[0].hInstance = hInstGlobal;
    psp[0].pszTemplate = MAKEINTRESOURCE(IDD_OPTIONS_DLG);
    psp[0].pszIcon = NULL;
    psp[0].pfnDlgProc = OptionsProc;
    psp[0].pszTitle = "Lock Options";
    psp[0].lParam = 0;

    psp[1].dwSize = sizeof(PROPSHEETPAGE);
    psp[1].dwFlags = PSP_USETITLE | PSP_HASHELP;
    psp[1].hInstance = hInstGlobal;
    psp[1].pszTemplate = MAKEINTRESOURCE(IDD_MENUOPTIONS_DLG);
    psp[1].pszIcon = NULL;
    psp[1].pfnDlgProc = MenuOptionsProc;
    psp[1].pszTitle = "Menu Options";
    psp[1].lParam = 0;

    psp[2].dwSize = sizeof(PROPSHEETPAGE);
    psp[2].dwFlags = PSP_USETITLE | PSP_HASHELP;
    psp[2].hInstance = hInstGlobal;
    psp[2].pszTemplate = MAKEINTRESOURCE(IDD_EDIT_PASSWORD_DLG);
    psp[2].pszIcon = NULL;
    psp[2].pfnDlgProc = EditPassWordProc;
    psp[2].pszTitle = "Edit Password";
    psp[2].lParam = 0;
   
    psh.dwSize = sizeof(PROPSHEETHEADER);
    psh.dwFlags = PSH_PROPSHEETPAGE | PSH_NOAPPLYNOW | PSH_USEICONID | PSH_USECALLBACK | PSH_MODELESS;
    psh.hwndParent = hwndOwner;
    psh.hInstance = hInstGlobal;
    psh.pszIcon = MAKEINTRESOURCE( IDI_ICON1 );
    psh.pszCaption = (LPSTR) "IconLock";
	psh.pfnCallback = (PFNPROPSHEETCALLBACK)PropSheetCallback;
    psh.nPages = sizeof(psp) / sizeof(PROPSHEETPAGE);
    psh.ppsp = (LPCPROPSHEETPAGE) &psp;

	
	
	hWndProp = (HWND)PropertySheet( &psh );
	
	if( !hWndProp )
		return 0;
	
	while(GetMessage(&msg, NULL, 0x00, 0x00))
	{
		if(hWndProp && (NULL == PropSheet_GetCurrentPageHwnd(hWndProp))) 
		{
			DestroyWindow(hWndProp);
			hWndProp = NULL;
			break;
		}

		//use PropSheet_IsDialogMessage instead of IsDialogMessage
		if( !PropSheet_IsDialogMessage(hWndProp, &msg) )
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	
	return bApplyChanges;
}


//=====================================================================
//  FUNCTION:		EditPassWordProc
//
//  PURPOSE:		Allows the user to enter or modify the password
//
//  PARAMETERS:	    standard
//
//  RETURN VALUE:	TRUE if a password was entered 
//
//  COMMENTS:       We allow the user to create a Question prompt to help remember
//                  the password.  Ex.  What is my pet's name?
//=====================================================================
BOOL EXPORTED CALLBACK EditPassWordProc( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	static PROPSHEETPAGE * ps;
	char szBuf[256];
	

	switch (uMsg)
	{
		case WM_INITDIALOG:	
			// save off the PROPSHEETPAGE information
			ps = (PROPSHEETPAGE *)lParam;
		    SendDlgItemMessage( hDlg, IDD_PASSWORD, EM_LIMITTEXT, MAX_PASSWORD_LEN-1, 0 );			
			SendDlgItemMessage( hDlg, IDD_QUESTION, EM_LIMITTEXT, MAX_PASSWORD_LEN-1, 0 );			

			Decrypt(szBuf, &iconlock.Password );
			SetDlgItemText( hDlg, IDD_PASSWORD, szBuf );
			SetDlgItemText( hDlg, IDD_CONFIRMPASSWORD, szBuf );
			SendDlgItemMessage( hDlg, IDD_PASSWORD, EM_SETMODIFY, 0, 0 );
				
			Decrypt( szBuf, &iconlock.Prompt );
			SetDlgItemText( hDlg, IDD_QUESTION, szBuf );
			
			return (TRUE);

		case WM_COMMAND:
		{
			WORD wNotifyCode = HIWORD(wParam); // notification code 
			WORD wID = LOWORD(wParam);         // item, control, or accelerator identifier 
			HWND hwndCtl = (HWND) lParam;      // handle of control 
			
			if( wID == IDD_PASSWORD && wNotifyCode == EN_CHANGE )
					SetDlgItemText( hDlg, IDD_CONFIRMPASSWORD, "" );
		}
		break;


		case WM_NOTIFY:
    		switch (((NMHDR FAR *) lParam)->code) 
    		{

				case PSN_SETACTIVE:
					break;

				case PSN_APPLY:
					GetDlgItemText( hDlg, IDD_PASSWORD, szBuf, sizeof( szBuf ) );
							
					Encrypt( szBuf, &iconlock.Password );
					
					GetDlgItemText( hDlg, IDD_QUESTION, szBuf, sizeof( szBuf ) );
					Encrypt( szBuf, &iconlock.Prompt );

					SaveDataFile( &iconlock, hMenuStrings ); 

 	           		SetWindowLong(hDlg,	DWL_MSGRESULT, TRUE);
					bApplyChanges = TRUE;
					break;

				case PSN_KILLACTIVE:
				{	
					BOOL bError = FALSE;
					char szPassword[MAX_PASSWORD_LEN], szConfirm[MAX_PASSWORD_LEN];
					GetDlgItemText( hDlg, IDD_PASSWORD, szPassword,sizeof(szPassword) );
					GetDlgItemText( hDlg, IDD_CONFIRMPASSWORD, szConfirm,sizeof(szConfirm) );
					if( lstrcmp( szPassword, szConfirm ) )
					{
						ShowError( IDS_BAD_PASSWORD );
						SetDlgItemText( hDlg, IDD_CONFIRMPASSWORD, "" );
						bError = TRUE;
					}

	           		SetWindowLong(hDlg,	DWL_MSGRESULT, bError );
					return 1;
				}
				

				case PSN_RESET:
					break;
				
				case PSN_HELP:
				{
					char szHelp[MAX_PATH];
					WinHelp( hDlg, GetHelpFile( szHelp ), HELP_CONTEXT, IDH_passwords );
				}
    	}
	}
	return (FALSE);   

}


//=====================================================================
//  FUNCTION:		OptionsProc
//
//  PURPOSE:		Iconlocks Options dialog
//
//  PARAMETERS:	    
//
//  RETURN VALUE:	
//
//  COMMENTS:       
//=====================================================================
BOOL EXPORTED CALLBACK OptionsProc( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	static PROPSHEETPAGE * ps;

	switch (uMsg)
	{
		case WM_INITDIALOG:	
		{
            DWORD dwMajor, dwMinor;
            BOOL bEnable = TRUE;

            // save off the PROPSHEETPAGE information
			ps = (PROPSHEETPAGE *)lParam;

			//check for active desktop
			
			if( !bActiveDesktop ) {
				EnableWindow( GetDlgItem( hDlg, IDC_EXPLORER ), FALSE );
				iconlock.byExplorerLocked = 0;
			}
			
			//Disable this option if active desktop is not installed or
            //if using NT check for IE5
			if(	IsWinNT( ) )
            {
				if( GetShell32Version( &dwMajor, &dwMinor, FALSE ) )
					if( dwMajor < 5 )
                        bEnable = FALSE;
            } else {
                bEnable = bActiveDesktop;
            }

            if( !bEnable )
            {
				EnableWindow( GetDlgItem( hDlg, IDC_LOCKCOMMONDLGS ), FALSE );
				iconlock.byLockCommonDlgs = 0;
			}


			CheckDlgButton( hDlg, IDC_EXPLORER           , iconlock.byExplorerLocked );
			CheckDlgButton( hDlg, IDC_LOCKTASKBAR        , iconlock.byTaskBarLocked );
			CheckDlgButton( hDlg, IDC_LOCKDATAFILES      , iconlock.byLockDataFiles );
			CheckDlgButton( hDlg, IDC_AUTORESTORE        , iconlock.byAutoRestore );
			CheckDlgButton( hDlg, IDC_LOCKFILESANDFOLDERS, iconlock.byLockFilesFolders );
			CheckDlgButton( hDlg, IDC_LOCKCOMMONDLGS     , iconlock.byLockCommonDlgs );

			CenterDialog( GetParent( hDlg ) );
			return (TRUE);
        }

		case WM_NOTIFY:
    		switch( ((NMHDR FAR *) lParam)->code ) 
    		{
				case PSN_SETACTIVE:
					break;

				case PSN_APPLY:
				{
 	           		SetWindowLong( hDlg, DWL_MSGRESULT, TRUE);
					iconlock.byExplorerLocked = IsDlgButtonChecked( hDlg, IDC_EXPLORER );
					iconlock.byTaskBarLocked  = IsDlgButtonChecked( hDlg, IDC_LOCKTASKBAR );
					iconlock.byLockDataFiles  = IsDlgButtonChecked( hDlg, IDC_LOCKDATAFILES );
					iconlock.byAutoRestore    = IsDlgButtonChecked( hDlg, IDC_AUTORESTORE );
                    iconlock.byLockCommonDlgs = IsDlgButtonChecked( hDlg, IDC_LOCKCOMMONDLGS );
			        iconlock.byLockFilesFolders = IsDlgButtonChecked( hDlg, IDC_LOCKFILESANDFOLDERS );
					SaveDataFile( &iconlock, hMenuStrings );
					bApplyChanges = TRUE;
				}
					break;

				case PSN_KILLACTIVE:
	           		SetWindowLong(hDlg,	DWL_MSGRESULT, FALSE);
					return 1;
					break;

				case PSN_RESET:
					break;

				case PSN_HELP:
				{
					char szHelp[MAX_PATH];
					WinHelp( hDlg, GetHelpFile( szHelp ), HELP_CONTEXT, IDH_lock_options );
				}

    		}	
			break;

	}
	return( FALSE );   

}



//=====================================================================
//  FUNCTION:		MenuOptionsProc
//
//  PURPOSE:		Iconlocks Menu Options dialog
//
//  PARAMETERS:	    
//
//  RETURN VALUE:	
//
//  COMMENTS:       
//=====================================================================
BOOL EXPORTED CALLBACK MenuOptionsProc( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	static PROPSHEETPAGE * ps;
	char szBuf[256];

	switch( uMsg )
	{
		case WM_INITDIALOG:	
		{
			int i;
			// save off the PROPSHEETPAGE information
			ps = (PROPSHEETPAGE *)lParam;

			if( hMenuStrings )
			{
				PMENUSTRINGS pMenuStrings = (PMENUSTRINGS)hMenuStrings;
				for( i = 0; i < iconlock.iNumberOfMenuStrings; i++ )
				{
					if( pMenuStrings[i].szMenuString[0] )
						SendDlgItemMessage( hDlg, IDC_DISABLED_LIST, LB_ADDSTRING, 0,(LPARAM)(LPSTR)pMenuStrings[i].szMenuString);
				}
			}

			FillDefaultsList( GetDlgItem( hDlg, IDC_MENUOPTION ) );

			SendDlgItemMessage( hDlg, IDC_MENUOPTION, EM_LIMITTEXT, MAX_MENUSTRING_LEN-1, 0 );			
			CheckAllMenuDefaults( hDlg );
			SendDlgItemMessage( hDlg, IDC_DISABLED_LIST, LB_SETCURSEL, 0, 0 );
			EnableRemoveButtons( hDlg );
			SetFocus( GetDlgItem( hDlg, IDC_MENUOPTION ) );
			
			///bAddState = TRUE;
			return( FALSE );
		}

		case WM_NOTIFY:
    		switch( ((NMHDR FAR *) lParam)->code ) 
    		{
				case PSN_SETACTIVE:
					break;

				case PSN_APPLY:
				{
					int i,iCount;
 	           		SetWindowLong( hDlg, DWL_MSGRESULT, TRUE);
					
					//this is needed to restore the DefaultID for property page
					SendMessage( hDlg, DM_SETDEFID, IDOK, 0L);

					iCount = SendDlgItemMessage(hDlg, IDC_DISABLED_LIST,LB_GETCOUNT, 0,0 );
					if( iCount == LB_ERR )
						break;
						
					iconlock.iNumberOfMenuStrings = iCount;
					if( hMenuStrings )
						GlobalFree( hMenuStrings );
					hMenuStrings = 0;

					if( iCount )
					{
						PMENUSTRINGS pMenuStrings;
						hMenuStrings = GlobalAlloc( GPTR, iCount * MAX_MENUSTRING_LEN );
						pMenuStrings = (PMENUSTRINGS)hMenuStrings;
						for( i = 0; i < iCount && hMenuStrings; i++ )
						{
							SendDlgItemMessage( hDlg, IDC_DISABLED_LIST,LB_GETTEXT, i,(LPARAM)(LPSTR)szBuf );
							strncpy( pMenuStrings[i].szMenuString, szBuf, MAX_MENUSTRING_LEN );
						}
					}
					SaveDataFile( &iconlock, hMenuStrings );
					bApplyChanges = TRUE;

				}	
					break;

				case PSN_KILLACTIVE:
	           		SetWindowLong(hDlg,	DWL_MSGRESULT, FALSE);
					//this is needed to restore the DefaultID for property page
					SendMessage( hDlg, DM_SETDEFID, IDOK, 0L);
					return 1;
					break;

				case PSN_RESET:
					break;
				
				case PSN_HELP:
				{
					char szHelp[MAX_PATH];
					WinHelp( hDlg, GetHelpFile( szHelp ), HELP_CONTEXT, IDH_context_menus );
				}
					break;


				
    		}

		case WM_VKEYTOITEM :
		{
			if( LOWORD(wParam) == VK_DELETE )
				SendMessage( hDlg, WM_COMMAND, IDC_REMOVE, 0 );
			else
				return( -1 );
		}
		break;

		case WM_COMMAND:
		{
			WORD wNotifyCode = HIWORD(wParam); // notification code 
			WORD wID = LOWORD(wParam);         // item, control, or accelerator identifier 
			HWND hwndCtl = (HWND) lParam;      // handle of control 

			
			switch( wID )
			{
				case IDC_ADD:
					if( wNotifyCode == BN_CLICKED )
					{				
						//( bAddState == TRUE )
						{
							if( GetDlgItemText(hDlg, IDC_MENUOPTION, szBuf,sizeof(szBuf) ) == 0 )
							{
								int i = SendDlgItemMessage( hDlg, IDC_MENUOPTION, CB_GETCURSEL,0,0 );
								if( i != CB_ERR )
									SendDlgItemMessage( hDlg, IDC_MENUOPTION,CB_GETLBTEXT, i, (LPARAM)szBuf );
							}
							
							if( szBuf[0] )
							{
								if( SendDlgItemMessage( hDlg, IDC_DISABLED_LIST,LB_FINDSTRINGEXACT, -1,(LPARAM)(LPSTR)szBuf ) == LB_ERR )
								{
									int i = SendDlgItemMessage(hDlg, IDC_DISABLED_LIST,LB_ADDSTRING, 0,(LPARAM)(LPSTR)szBuf );
									SendDlgItemMessage( hDlg, IDC_DISABLED_LIST, LB_SETCURSEL, i, 0 );
								}
								SetDlgItemText( hDlg, IDC_MENUOPTION, "" );
								CheckAllMenuDefaults( hDlg );
								SetFocus( GetDlgItem( hDlg, IDC_MENUOPTION ) );
							}
							 
						} 
					}
					break;
				
				case IDC_REMOVE:
					{
					int iSelected = SendDlgItemMessage( hDlg, IDC_DISABLED_LIST, LB_GETCURSEL, 0, 0 );
					if( iSelected != LB_ERR ) {
						
                        SendDlgItemMessage( hDlg, IDC_DISABLED_LIST, LB_GETTEXT, iSelected, (LPARAM)(LPSTR)szBuf );                    
                        
                        SendDlgItemMessage( hDlg, IDC_DISABLED_LIST, LB_DELETESTRING, iSelected, 0 );
						SendDlgItemMessage( hDlg, IDC_DISABLED_LIST, LB_SETCURSEL, max(0,iSelected-1), 0 );
                        CheckAllMenuDefaults( hDlg );
                        SetDlgItemText( hDlg, IDC_MENUOPTION, szBuf );
                    }
					
					}
					break;

				case IDC_DEFAULTS:
					AddDefaults( hDlg );
					CheckAllMenuDefaults( hDlg );
					break;

				case IDC_REMOVE_ALL:
					SendDlgItemMessage( hDlg, IDC_DISABLED_LIST, LB_RESETCONTENT, 0, 0 );
					CheckAllMenuDefaults( hDlg );
					break;

				//Force the Add button to be default -- this way the Enter key will default to Add			
				case IDC_MENUOPTION:
					if( wNotifyCode == CBN_SETFOCUS || wNotifyCode == CBN_EDITCHANGE )
					{
						SendDlgItemMessage( hWndProp, IDOK,  BM_SETSTYLE, BS_PUSHBUTTON, (LONG)TRUE);	//1.1
						SendMessage( hDlg, DM_SETDEFID, IDC_ADD, 0L);
						SendDlgItemMessage( hDlg, IDC_ADD, BM_SETSTYLE, BS_DEFPUSHBUTTON, (LONG)TRUE);
						EnableRemoveButtons( hDlg );
					}	
					break;
						
				case IDC_DISABLED_LIST:
					switch( wNotifyCode	)
					{

						case LBN_SELCHANGE:
							EnableRemoveButtons( hDlg );
							break;

						case LBN_DBLCLK:
							{
								WPARAM i = SendMessage( hwndCtl, LB_GETCURSEL, 0, 0 );
								if( i != LB_ERR )
								{
									SendMessage( hwndCtl, LB_GETTEXT, i, (LPARAM)(LPSTR)szBuf );
									SetDlgItemText( hDlg, IDC_MENUOPTION, szBuf );
									SetFocus( GetDlgItem( hDlg, IDC_MENUOPTION ) ); 
								}
							}
							break;
					}
					break;

				default:
					return( FALSE );
			}
		
			EnableRemoveButtons( hDlg );
			return( TRUE );
		}
			
	}
	
	return( FALSE );   
}





void FillDefaultsList( HWND hCtrl )
{
char szBuf[2048];
LPSTR pStart,pEnd;

	if( LoadString( hInstGlobal, IDS_DEFAULTS, szBuf, sizeof( szBuf ) ) )
	{
		pStart = szBuf;
		while( pStart )
		{
			pEnd = strchr( pStart, ',' );
			if( pEnd )
			{
				*pEnd = 0;
				SendMessage( hCtrl, CB_INSERTSTRING, -1, (LPARAM)(LPCSTR) pStart );
			} else {
				break;
			}
			pStart = ++pEnd;		

		}

	}
}

void AddDefaults( HWND hDlg )
{
char szBuf[2048];
LPSTR pStart,pEnd;
HWND hDisableList =	GetDlgItem( hDlg, IDC_DISABLED_LIST );


	if( LoadString( hInstGlobal, IDS_DEFAULTS, szBuf, sizeof( szBuf ) ) )
	{
		pStart = szBuf;
		while( pStart )
		{
			pEnd = strchr( pStart, ',' );
			if( pEnd )
			{
				*pEnd = 0;
				
		 		if( SendMessage( hDisableList, LB_FINDSTRINGEXACT, -1, (LPARAM)(LPCSTR) pStart ) == LB_ERR )
					SendMessage( hDisableList, LB_ADDSTRING, 0, (LPARAM)(LPCSTR)pStart );

			} else {
				break;
			}
			pStart = ++pEnd;		

		}

	}
}


void EnableRemoveButtons( HWND hDlg )
{
char szBuf[10];
	
	HWND hList = GetDlgItem( hDlg, IDC_DISABLED_LIST);
	int  iSelected = SendMessage( hList, LB_GETCURSEL, 0, 0 );

	EnableWindow( GetDlgItem( hDlg, IDC_REMOVE ), iSelected != LB_ERR );

	EnableWindow( GetDlgItem( hDlg, IDC_REMOVE_ALL ),
			SendMessage( hList, LB_GETCOUNT, 0, 0 ) > 0 );

	EnableWindow( GetDlgItem( hDlg, IDC_ADD ),
			      GetDlgItemText( hDlg, IDC_MENUOPTION, szBuf,sizeof(szBuf) ) > 0  ||
				  SendDlgItemMessage( hDlg, IDC_MENUOPTION, CB_GETCURSEL,0,0 ) != CB_ERR );
}




void  CenterDialog(HWND hDlg)
{    
	RECT rRect;

    GetWindowRect(hDlg, &rRect);
    OffsetRect(&rRect, -rRect.left, -rRect.top);
    MoveWindow(hDlg, ((GetSystemMetrics(SM_CXSCREEN) - rRect.right ) / 2 + 4) & ~7,
         (GetSystemMetrics(SM_CYSCREEN) - rRect.bottom) / 2, rRect.right, rRect.bottom, 0);
	return; 
	
	
}

void CALLBACK PropSheetCallback(HWND hwndPropSheet, UINT uMsg, LPARAM lParam)
{
	switch(uMsg)
	{
		//called before the dialog is created, remove the system menu
		case PSCB_PRECREATE:
		{
			LPDLGTEMPLATE  lpTemplate = (LPDLGTEMPLATE)lParam;
			lpTemplate->style &= ~DS_CONTEXTHELP;
		}
		break;

		case PSCB_INITIALIZED:
			CenterDialog( hwndPropSheet );
		break;

	}
}


//check to see if all the default menu items are already in the disabled list
//if so disable the button
void CheckAllMenuDefaults( HWND hDlg )
{
char szBuf[2048];
LPSTR pStart,pEnd;
HWND hDisableList =	GetDlgItem( hDlg, IDC_DISABLED_LIST );
HWND hMenuList = GetDlgItem( hDlg, IDC_MENUOPTION );
BOOL bEnable = FALSE;

	SendMessage( hMenuList, CB_RESETCONTENT, 0, 0 );
	if( LoadString( hInstGlobal, IDS_DEFAULTS, szBuf, sizeof( szBuf ) ) )
	{
		pStart = szBuf;
		while( pStart )
		{
			pEnd = strchr( pStart, ',' );
			if( pEnd )
			{
				*pEnd = 0;
		 		if( SendMessage( hDisableList, LB_FINDSTRINGEXACT, -1, (LPARAM)(LPCSTR) pStart ) == LB_ERR )
				{
					bEnable = TRUE;
					SendMessage( hMenuList, CB_INSERTSTRING, -1, (LPARAM)(LPCSTR) pStart );
				}
			} else {
				break;
			}
			pStart = ++pEnd;		

		}

	}

	EnableWindow( GetDlgItem( hDlg, IDC_DEFAULTS ), bEnable );
}




