/////////////////////////////////////////////////////////////////////////////
// ThreadData
//
//Different shell object views, folders, my computer, explorer, internet explorer
//may share instances of each other -- this varies upon version, platform, configuration
//and so on.
//Because we do not have a 'this' pointer for static callbacks and hooks, we need
//to maintain an array that contains the threadID and the 'this' pointer.  Whenever
//a static function is called, we can then lookup the 'this' pointer.
//
// 
//
//    Copyright (c) 1999 Ziff Davis, Inc. All rights reserved
//    First appeared in PC Magazine, US Edition, August, 1999
//                Programmed by Mark Sweeney
//


#include "Stdafx.h"
#include "hookexplorer.h"
#include "ThreadData.h"

static HGLOBAL hProcessList = 0;
static int iProcessListCount = 0;

///////////////////////////////////////////////////////////////////////////////
//
void _cdecl DebugOut_CompiledOutOfReleaseBuild( TCHAR* format, ... )
{
	#ifdef _DEBUG

    static TCHAR szBuf[1024];
    static TCHAR sz[30];
	static va_list argptr;

	{
		va_start( argptr, format );
		wvsprintf( szBuf, format, argptr );
		va_end( argptr);
		wsprintf(sz, _T("%X  (LockHlpr) "), GetCurrentThreadId() );
		OutputDebugString( sz );
		OutputDebugString( szBuf );
		OutputDebugString( _T("\n\r") );
	}

	#endif

}


/////////////////////////////////////////////////////////////////////////////
// 
BOOL AddToProcessList( CHookExplorer* pThis )
{
	DWORD dwSize = (iProcessListCount+1) * sizeof( PROCESS_LIST );
	if( hProcessList == 0 )
	{
		hProcessList = GlobalAlloc( GHND, dwSize );
	} else {
		hProcessList = GlobalReAlloc( hProcessList, dwSize, GMEM_ZEROINIT );
		if( !hProcessList )
			DEBUGOUT( _T("Realloc failed!!!!!!!! %i %x"), iProcessListCount, dwSize );
	}

	if( hProcessList )
	{
		LPPROCESS_LIST pList = (LPPROCESS_LIST)GlobalLock( hProcessList );
		pList[iProcessListCount].dwThread = GetCurrentThreadId();
		pList[iProcessListCount].pThis = pThis;
		GlobalUnlock( hProcessList );
		iProcessListCount++;
		DEBUGOUT( _T("AddToProcessList %u %X %X"), iProcessListCount,GetCurrentThreadId(), pThis );
		
	} else {
		iProcessListCount = 0;
	}

	return( iProcessListCount ); 
}

/////////////////////////////////////////////////////////////////////////////
// 
void RemoveThreadFromProcessList(  )
{
int i;
	
	if( hProcessList )
	{
		//remove entry from list
		LPPROCESS_LIST pList = (LPPROCESS_LIST)GlobalLock( hProcessList );
		for( i=0; i<iProcessListCount; i++ ) 
		{
			if( pList[i].dwThread == GetCurrentThreadId() )
			{
				pList[i].dwThread = 0;
				break;		
			} 
		}
		
		//check to see if the list is empty
		for( i=0; i<iProcessListCount; i++ ) 
		{
			if( pList[i].dwThread )
				break;
		}
		GlobalUnlock( hProcessList );

		//free the list
		if( i == iProcessListCount )
		{
			GlobalFree( hProcessList );
			iProcessListCount = 0;
			hProcessList = 0;
		}

	}
}


/////////////////////////////////////////////////////////////////////////////
// 
CHookExplorer* Get_pThis( ) 
{
	CHookExplorer* pThis = 0;
			
	if( hProcessList )
	{
		LPPROCESS_LIST pList = (LPPROCESS_LIST)GlobalLock( hProcessList );
		for( int i=0; i<iProcessListCount; i++ ) 
		{
			if( pList[i].dwThread == GetCurrentThreadId() )
			{
				pThis = pList[i].pThis;
				break;		
			} 
		}
		GlobalUnlock( hProcessList );
	}

	return( pThis );
}


/////////////////////////////////////////////////////////////////////////////
// 
#ifdef _DEBUG
void DumpList( )
{
	if( hProcessList )
	{
		LPPROCESS_LIST pList = (LPPROCESS_LIST)GlobalLock( hProcessList );
		for( int i = 0; i < iProcessListCount; i++ )
			DEBUGOUT( _T("DumpList -- %u %X %x"), i, pList[i].dwThread, pList[i].pThis );

		GlobalUnlock( hProcessList );
	}
}
#endif
