/////////////////////////////////////////////////////////////////////////////
// CallBack.cpp : Implementation of CCallBack
//
//
//    Copyright (c) 1999 Ziff Davis, Inc. All rights reserved
//    First appeared in PC Magazine, US Edition, August, 1999
//                Programmed by Mark Sweeney
//




#include "stdafx.h"
#include "LockCopy.h"
#include "CallBack.h"
#include "..\exe\iconlock.h"

HWND FindWindowClass( HWND hWnd, PTCHAR pClassName );

typedef struct {
	PTCHAR  pClassname;
	HWND   hWndFound;
} ENUM_CLASS, *LPENUM_CLASS; 


extern CComModule _Module;


/////////////////////////////////////////////////////////////////////////////
// CCallBack
//
// The callback is notified whenever a file operation is about to take place 
// on a folder.   If the folder is in the Desktop file system then disallow
// the operation and display a message box.
//
//
UINT CCallBack::CopyCallback(HWND hwnd, UINT wFunc, UINT wFlags,
                            LPCSTR pszSrcFile,  DWORD dwSrcAttribs,
                            LPCSTR pszDestFile, DWORD dwDestAttribs)
{
WORD wReturn = IDYES;

	switch( wFunc )
	{
		case FO_MOVE:
		case FO_RENAME:
		case FO_COPY:
			if( IsDesktopFolder(pszSrcFile) || IsDesktopFolder(pszDestFile) )
				wReturn = IDNO;			
			break;			

		case FO_DELETE:
			if( IsDesktopFolder(pszSrcFile) )
				wReturn = IDNO;
			break;
	}

   if( wReturn != IDYES )
   {
   		TCHAR szMsg[MAX_PATH * sizeof(TCHAR) * 2 ];
		if( IsDesktopLocked( ) )
		{
			wsprintf( szMsg, "Access to the folder %s is restricted by IconLock.\nPlease select another location.", pszSrcFile );
			MessageBox( GetDesktopWindow(), szMsg, _T( SZ_APPNAME ), MB_OK | MB_ICONEXCLAMATION | MB_SETFOREGROUND );  
		} else {
			wReturn = IDYES;
		}
   } 
   
  
   return wReturn;
}

/////////////////////////////////////////////////////////////////////////////
//	IsDesktopLocked
//
//	Send a private message to ProgMag.  The LockDLL.dll will pick up the 
//  message and return the current lock state.
//
BOOL CCallBack::IsDesktopLocked( )
{

BOOL bLockFlags;

	if( m_PrivateMessage == 0 )
		m_PrivateMessage = RegisterWindowMessage( _T( SZ_APPNAME ) );

		HWND hwnd = FindWindow( _T("Progman"), NULL );
		if( hwnd ) {
			HWND hShell = FindWindowClass( hwnd, _T("SHELLDLL_DefView") );
			if( hShell )
				bLockFlags = SendMessage( hShell, m_PrivateMessage, 0 , 0 );

			DEBUGOUT( _T("PrivateMessage returned %x LockCopy Flags = %x"), hShell, bLockFlags );
			
			return( ( bLockFlags &= FILESFOLDERS_LOCKED) == FILESFOLDERS_LOCKED );
		}
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// IsDesktopFolder
//
// Returns TRUE if pDir is in the Desktop file system
//
BOOL CCallBack::IsDesktopFolder( LPCSTR pDir )
{
char szDir[MAX_PATH*sizeof(TCHAR)];

	GetDesktopFolders( );
	
	_tcsncpy( szDir, pDir,sizeof(szDir) ); 
	_tcsupr( szDir );
	
	if( _tcsstr( szDir, m_szDesktopDir ) )
		return TRUE;

	if( _tcsstr( szDir, m_szDesktopDirectory) )
		return TRUE;

	if( _tcsstr( szDir, m_szCommonDesktop ) )
		return TRUE;

	return FALSE;
}						

/////////////////////////////////////////////////////////////////////////////
//	GetDesktopFolders
//
//	Get the directories that comprise the Desktop file system
//
void CCallBack::GetDesktopFolders( )
{
LPITEMIDLIST pidl;   
LPMALLOC pMalloc;
char sz[MAX_PATH*sizeof(TCHAR)];
static BOOL bDone = 0;

	if( !bDone )
	{
		if( SUCCEEDED(SHGetMalloc( &pMalloc )) ) 
		{
			SHGetSpecialFolderLocation( NULL, CSIDL_DESKTOP, &pidl );
			SHGetPathFromIDList( pidl, sz );
			
			if( *sz ) {
				_tcsupr( sz );
				_tcsncpy( m_szDesktopDir, sz, sizeof(m_szDesktopDir) ); 
			}

			SHGetSpecialFolderLocation( NULL, CSIDL_DESKTOPDIRECTORY, &pidl );
			SHGetPathFromIDList( pidl, sz );

			if( *sz ) {
				_tcsupr( sz );
				_tcsncpy( m_szDesktopDirectory, sz, sizeof(m_szDesktopDirectory) ); 
			}


			SHGetSpecialFolderLocation( NULL, CSIDL_COMMON_DESKTOPDIRECTORY, &pidl );
			SHGetPathFromIDList( pidl, sz );

			if( *sz ) {
				_tcsupr( sz );
				_tcsncpy( m_szCommonDesktop, sz,sizeof(m_szCommonDesktop) ); 
			}

			pMalloc->Free( pidl );
			pMalloc->Release( );
		}
	
		bDone = TRUE;
	}

}


/////////////////////////////////////////////////////////////////////////////
// EnumProc -- Searches for an hWnd having the classname in lParam
//
//  
BOOL CALLBACK EnumProc( HWND hWnd, LPARAM lParam )
{
TCHAR sz[256];
LPENUM_CLASS pEnumClass = (LPENUM_CLASS) lParam;
 	
	GetClassName( hWnd, sz, sizeof(sz) );

	if( _tcsicmp( sz, pEnumClass->pClassname ) == 0 ) 
	{
		pEnumClass->hWndFound = hWnd;
		return( FALSE );
	}

	return( TRUE );
}


/////////////////////////////////////////////////////////////////////////////
//  FindWindowClass -- hWnd is the starting window to enumerate, pClassname
//                     is the class to locate.  Returns the handle of the 
//					   window.
//
HWND FindWindowClass( HWND hWnd, PTCHAR pClassName )
{
	ENUM_CLASS enumclass;

	enumclass.pClassname = pClassName;
	enumclass.hWndFound = 0;
	EnumChildWindows( hWnd, EnumProc, (LPARAM)&enumclass );
	return( enumclass.hWndFound );
}
