/////////////////////////////////////////////////////////////////////////////
// LockFolder.cpp : Implementation of CFolder
//
//  Because the desktop namespace has a Folder with this CLSID this extension  
//	will be loaded whenever the GetAttributesOf() interface is called.
//
//    Copyright (c) 1999 Ziff Davis, Inc. All rights reserved
//    First appeared in PC Magazine, US Edition, August, 1999
//                Programmed by Mark Sweeney
//


#include "stdafx.h"
#include "LockDlgs.h"
#include "ExDisp.h"
#include "Shlobj.h"
#include "LockFolder.h"
#include "LockFold.h"
#include "..\exe\iconlock.h"
#include <Winver.h>



extern BOOL bOKtoUnLoad;
extern HINSTANCE g_hInstance;
UINT g_cRefThisDll;

typedef struct {
	LPCTSTR pClassname;
	HWND   hWndFound;
} ENUM_CLASS, *LPENUM_CLASS; 

WNDPROC fpShellProc;
LRESULT CALLBACK WindowProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
LRESULT CALLBACK ShellProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
HWND hShellWnd;
HWND hLockWnd;
HWND CreateLockWindow( HWND hWnd );
SubclassShellDLL( HWND hParent );
void EnableDefaultButton( HWND hTop, BOOL bEnable );

 
/////////////////////////////////////////////////////////////////////////////
//
CFolder::CFolder()
{
	m_cRef = 1;
    g_cRefThisDll++;
}

/////////////////////////////////////////////////////////////////////////////
//
CFolder::~CFolder()
{
	g_cRefThisDll--;
}


/////////////////////////////////////////////////////////////////////////////
// QueryInterface()
// returns pointer to the specified interface
STDMETHODIMP CFolder::QueryInterface( REFIID riid, LPVOID FAR *ppv )
{
	if( IsEqualIID(riid, IID_IUnknown) ) 
	{
		*ppv =  this;
        m_cRef++;
        return NOERROR;
    }
    else 
	if( IsEqualIID(riid, IID_IShellFolder) ) 
	{
        *ppv = (IShellFolder *) this;
        m_cRef++;
        return NOERROR;
    }
    else 
	if( IsEqualIID(riid, IID_IPersistFolder) ) 
	{
        *ppv = (IPersistFolder *) this;
        m_cRef++;
        return NOERROR;
    }
    else 
	{
        *ppv = NULL;
        return ResultFromScode( E_NOINTERFACE );
    }
}



/////////////////////////////////////////////////////////////////////////////
//	AddRef()
//	increase the reference count for the server 
STDMETHODIMP_(ULONG) CFolder::AddRef()
{
    return ++m_cRef;
}


/////////////////////////////////////////////////////////////////////////////
//	Release()
//	decrease the reference count for the server 
STDMETHODIMP_(ULONG) CFolder::Release()
{
    if( --m_cRef==0 )
	{
        delete this;
		return 0;
	}
    return m_cRef;
}


/////////////////////////////////////////////////////////////////////////////
//	EnumObjects()
STDMETHODIMP CFolder::EnumObjects( HWND hwndOwner, DWORD grfFlags, LPENUMIDLIST *ppEnum )
{
	return E_NOTIMPL;
}


/////////////////////////////////////////////////////////////////////////////
//	CompareIDs()
STDMETHODIMP CFolder::CompareIDs( LPARAM lParam, LPCITEMIDLIST pidl1, LPCITEMIDLIST pidl2 )
{
	return E_NOTIMPL;
}


/////////////////////////////////////////////////////////////////////////////
//	CreateViewObject
STDMETHODIMP CFolder::CreateViewObject( HWND hwndOwner, REFIID riid, LPVOID FAR* ppvObj )
{
	if( riid != IID_IShellView )
		return E_NOINTERFACE;
	
	return NOERROR;
}


/////////////////////////////////////////////////////////////////////////////
//	GetAttributesOf()
//  Returns the attribute of some objects
STDMETHODIMP CFolder::GetAttributesOf( UINT cidl, 
   LPCITEMIDLIST FAR* apidl, ULONG FAR* rgfInOut )
{

	*rgfInOut = SFGAO_FILESYSTEM ;
	
	//Because we have placed a folder on the Desktop for this CLSID
	//and because the CLSID has the CallForAttributes registry entry
	//the shell will call this interface whenever the user 
	//navigates to a Desktop directory. Subclass the window if 
	//it is a common dialog.
	SubclassShellDLL( GetForegroundWindow() );
	return S_OK;
}



/////////////////////////////////////////////////////////////////////////////
//	GetUIObjectOf()
// Returns an interface to perform UI actions
STDMETHODIMP CFolder::GetUIObjectOf( HWND hwndOwner, 
   UINT cidl, LPCITEMIDLIST FAR* apidl, REFIID riid, 
   UINT FAR* prgfInOut, LPVOID FAR* ppvObj )
{

	return E_NOINTERFACE;
}


/////////////////////////////////////////////////////////////////////////////
//	GetDisplayNameOf()
//  Returns the name to display

STDMETHODIMP CFolder::GetDisplayNameOf( LPCITEMIDLIST pidl, DWORD dwReserved, LPSTRRET lpName )
{
	return E_NOTIMPL;
}



/////////////////////////////////////////////////////////////////////////////
//	ParseDisplayName()
STDMETHODIMP CFolder::ParseDisplayName( HWND hwndOwner,
   LPBC pbc, LPOLESTR lpszDisplayName, ULONG FAR* pchEaten, 
   LPITEMIDLIST * ppidl, ULONG *pdwAttributes)
{
	return E_NOTIMPL;
}

/////////////////////////////////////////////////////////////////////////////
//	BindToObject()
STDMETHODIMP CFolder::BindToObject( LPCITEMIDLIST pidl, 
   LPBC pbc, REFIID riid, LPVOID FAR* ppvObj )
{
	return E_NOTIMPL;
}


/////////////////////////////////////////////////////////////////////////////
//	BindToStorage()
STDMETHODIMP CFolder::BindToStorage( LPCITEMIDLIST pidl, 
   LPBC pbc, REFIID riid, LPVOID FAR* ppvObj )
{
	return E_NOTIMPL;
}


/////////////////////////////////////////////////////////////////////////////
//	SetNameOf()
STDMETHODIMP CFolder::SetNameOf( HWND hwndOwner, LPCITEMIDLIST pidl,
	LPCOLESTR lpszName, DWORD dwReserved, LPITEMIDLIST FAR* ppidlOut )
{
	return E_NOTIMPL;
}


/////////////////////////////////////////////////////////////////////////////
//	GetClassID()
//  Returns our CLSID 
STDMETHODIMP CFolder::GetClassID( LPCLSID lpClassID )
{
	*lpClassID = CLSID_LockFolder;
	return NOERROR;
}



/////////////////////////////////////////////////////////////////////////////
//	Initialize()
STDMETHODIMP CFolder::Initialize( LPCITEMIDLIST pidl )
{
	return S_OK;
}



/////////////////////////////////////////////////////////////////////////////
//	EnumFolderProc()
BOOL CALLBACK EnumFolderProc( HWND hWnd, LPARAM lParam )
{
TCHAR sz[256];
LPENUM_CLASS pEnumClass = (LPENUM_CLASS) lParam;
 	
	GetClassName( hWnd, sz, sizeof(sz)*sizeof(TCHAR) );
	
	if( _tcsicmp( sz, pEnumClass->pClassname ) == 0 ) 
	{
		pEnumClass->hWndFound = hWnd;
		return( FALSE );
	}

	return( TRUE );
}

/////////////////////////////////////////////////////////////////////////////
//	GetShellDefView()
HWND GetShellDefView( HWND hParent )
{
	if( hParent )
	{
		ENUM_CLASS enumclass;
		enumclass.pClassname = _T("SHELLDLL_DefView");	
		enumclass.hWndFound = 0;
		EnumChildWindows( hParent, (WNDENUMPROC)EnumFolderProc, (LPARAM)&enumclass ); 
		return( enumclass.hWndFound );
	}	
	return 0;
}


/////////////////////////////////////////////////////////////////////////////
//	GetListView()
HWND GetListView( HWND hParent )
{
	if( hParent )
	{
		ENUM_CLASS enumclass;
		enumclass.pClassname = _T("SysListView32");
		enumclass.hWndFound = 0;
		EnumChildWindows( hParent, (WNDENUMPROC)EnumFolderProc, (LPARAM)&enumclass ); 
		return( enumclass.hWndFound );
	}	
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
//	EnumComboProc()
BOOL CALLBACK EnumComboProc( HWND hWnd, LPARAM lParam )
{
TCHAR sz[256];
LPENUM_CLASS pEnumClass = (LPENUM_CLASS) lParam;
 	
	GetClassName( hWnd, sz, sizeof(sz)*sizeof(TCHAR) );
	
	if( _tcsicmp( sz, pEnumClass->pClassname ) == 0 ) 
	{
		GetWindowText( hWnd, sz, sizeof(sz)*sizeof(TCHAR) );
	
		if( _tcsicmp( sz, _T("Desktop") ) == 0  )
		{
			pEnumClass->hWndFound = (HWND)TRUE;
			return( FALSE );
		}
	}

	return( TRUE );
}

/////////////////////////////////////////////////////////////////////////////
//
BOOL ExamineComboBoxes( HWND hWnd )
{
		ENUM_CLASS enumclass;
		enumclass.pClassname = _T("ComboBox");
		enumclass.hWndFound = 0;
		EnumChildWindows( hWnd, (WNDENUMPROC)EnumComboProc, (LPARAM)&enumclass ); 
		return( (BOOL)enumclass.hWndFound );
}

/////////////////////////////////////////////////////////////////////////////
//	CreateLockWindow()
//	Creates a window to overlay the common dialog listview
//
HWND CreateLockWindow( HWND hWnd )
{
	//if our window class has not been registered, then do so now
	TCHAR CLASS_NAME[] = _T("LockDlgsExt");
	WNDCLASS wc;
	
	if(!GetClassInfo( g_hInstance, CLASS_NAME, &wc))
	{
	   ZeroMemory( &wc, sizeof(wc));
	   wc.style          = CS_HREDRAW | CS_VREDRAW;
	   wc.lpfnWndProc    = (WNDPROC)WindowProc;
	   wc.cbClsExtra     = 0;
	   wc.cbWndExtra     = 0;
	   wc.hInstance      = g_hInstance;
	   wc.hIcon          = NULL;
	   wc.hCursor        = LoadCursor(NULL, IDC_ARROW);
	   wc.hbrBackground  = (HBRUSH)(COLOR_WINDOW + 1);
	   wc.lpszMenuName   = NULL;
	   wc.lpszClassName  = CLASS_NAME;
   
	   if(!RegisterClass(&wc))
		  return 0;
	}

	hLockWnd = 	CreateWindowEx( 0,
								CLASS_NAME,
								NULL,
								WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_BORDER,
								0,0,0,0,
								hWnd,
								(HMENU)12345,
								g_hInstance,
								(LPVOID)0);
	
	return( hLockWnd );

}

/////////////////////////////////////////////////////////////////////////////
//	DisableButtonProc()
//
//  Disables or enables the toolbar and push button on 
//  the common dialog.
//
BOOL CALLBACK DisableButtonProc( HWND hWnd, LPARAM lParam )
{
TCHAR sz[256];
 	
	GetClassName( hWnd, sz, sizeof(sz)*sizeof(TCHAR) );

	if( _tcsicmp( sz, _T("ToolbarWindow32") ) == 0 ) 
	{
		//EnableWindow( hWnd, (BOOL)lParam );

		int i, icount;
		TBBUTTON tb;
		icount = SendMessage( hWnd, TB_BUTTONCOUNT, 0, 0 );
		
		for( i = 2; i < icount; i++ )
		{
			if( SendMessage( hWnd, TB_GETBUTTON, i, (LPARAM)&tb ) )
			{
				BOOL bEnable;
				if( lParam )
					bEnable = TRUE;
				else
					bEnable = FALSE;
				SendMessage( hWnd, TB_ENABLEBUTTON, (WPARAM)tb.idCommand, MAKELONG(bEnable,0) );
			}
		}


	}

	if( _tcsicmp( sz, _T("Button") ) == 0 ) 
	{
		if( GetWindowLong( hWnd, GWL_ID ) == IDOK )
		{
			EnableWindow( hWnd, (BOOL)lParam );
		}
	}
	return( TRUE );
}

/////////////////////////////////////////////////////////////////////////////
//	EnableDefaultButton()
//	enable or disable common dialog toolbar and push button
//
void EnableDefaultButton( HWND hTop, BOOL bEnable )
{
	EnumChildWindows( hTop, (WNDENUMPROC)DisableButtonProc, (LPARAM)bEnable); 
}

/////////////////////////////////////////////////////////////////////////////
//	GetTopMostWindow()
//	Searches for the true parent of the child window
//
HWND GetTopMostWindow( HWND hWnd )
{
	while( TRUE )
	{
		if( GetParent( hWnd ) && GetWindow( hWnd, GW_OWNER ) == 0 )
  		{
			hWnd = GetParent( hWnd );	
		} else {
			break;
		}
	}

	return( hWnd );
}


/////////////////////////////////////////////////////////////////////////////
//	SubclassShellDLL()
//
//	This is where we subclassthe common dialog.  The function checks to see
//  if the window which caused the shell to call the interface for GetAttributesOf
//  is ProgMan (the Desktop), or Explorer.  If so, it then don't subclass it -- note
//  if you remove the check for ExploreWClass below, then the right pane in Explorer
//  will be overlayed and disabled with our window.
//
//  If we do subclass the window then set the global bOKtoUnLoad to FALSE so
//  that our DLL does not get unloaded.
//
//

SubclassShellDLL( HWND hParent )
{
	
	//already subclassed?
	if( fpShellProc )
	{
		return 0;
	}

	//check to see if our parent is ProgMan, if so don't create our window
	HWND hTop = GetTopMostWindow( hParent );
	{
		TCHAR szClass[80];
		if( hTop )
		{
			GetClassName( hTop, szClass, sizeof(szClass)*sizeof(TCHAR) );
			
			if( !_tcsicmp( szClass, _T("Progman") )       ||
				!_tcsicmp( szClass, _T("ExploreWClass") ) ||
				!_tcsicmp( szClass, _T("CabinetWClass") ) )
			{
				bOKtoUnLoad = TRUE;
				return 0;
			}
		}
	}

	

	//Find the SHELLDLL_DefView child window
	HWND hWnd = GetShellDefView( hParent );

	if( hWnd )
	{
		if( ExamineComboBoxes( hParent ) == FALSE )
		{
				bOKtoUnLoad = TRUE;
				return 0;
		}

		bOKtoUnLoad = FALSE;
		CreateLockWindow( hWnd );
		
		//set global hShellWnd 
		hShellWnd = hWnd;

		//subclass the dialog
		fpShellProc = (WNDPROC)SetWindowLong( hWnd, GWL_WNDPROC, (LONG)ShellProc );
		
		//disable the open or save button and the toolbar
		EnableDefaultButton( GetParent(hWnd), FALSE );		
		InvalidateRect( hWnd, NULL, TRUE );
		
	} else {
		//no SHELLDLL_DefView, its not a common dialog, unload
		bOKtoUnLoad = TRUE;
	}

	return 1;
}


/////////////////////////////////////////////////////////////////////////////
//	ShellProc()
//
//	Window proc for the subclassed dialog.  There is not much we can do here
//  since Win9x and NT work so differently -- actually, this subclass is only
//  required for NT.
//  Watch for sizing and erase messages so that we can paint our window.
//
LRESULT CALLBACK ShellProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{

	switch( msg )
	{

		case WM_DESTROY:
			DEBUGOUT( "WM_DESTROY ShellProc" );
			break;
		
		//for Save button in common dialog -- don't get this with all dlgs
		case WM_COMMAND:
			return 0;
		
		case WM_INITMENU:
			DestroyMenu( (HMENU) wParam );
			return 0;

		case WM_NOTIFY:
			return 0;

		case WM_LBUTTONDOWN:
		case WM_RBUTTONDOWN:
			break;

		case WM_ERASEBKGND:
		{
			RECT rc;
			GetClientRect( hWnd, &rc );
			SetWindowPos( hLockWnd, HWND_TOP, 0,0,rc.right,rc.bottom, SWP_NOACTIVATE );
		}
		break;

		case WM_SIZE:
		{
			int nWidth = LOWORD(lParam);  // width of client area 
			int nHeight = HIWORD(lParam); // height of client area 
			SetWindowPos( hLockWnd, HWND_TOP, 0,0,nWidth, nHeight, SWP_NOACTIVATE );
			break;
		}
		
	}

	if( fpShellProc )
		return CallWindowProc( fpShellProc, hWnd, msg, wParam, lParam );
	return 0;
}



/////////////////////////////////////////////////////////////////////////////
//	WindowProc()
//
//	Window proc for our temporary window.  Disable certain messages and
//  paint our overlay window.   When we receive the WM_DESTROY, unsubclass
//  the dialog and set bOKtoUnLoad to TRUE so our DLL will be unloaded.
//	Note: it may take several seconds before COM will actually unload the DLL.
//
LRESULT CALLBACK WindowProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	
	switch( msg )
	{
		case LVM_GETSELECTEDCOUNT:
			return 0;

		case WM_DESTROY:
		{
			DEBUGOUT( "WM_DESTROY WindowProc" );
			if( hShellWnd && IsWindow( hShellWnd ) )
				SetWindowLong( hShellWnd, GWL_WNDPROC, (LONG)fpShellProc );
			fpShellProc = NULL;
			bOKtoUnLoad = TRUE;
			EnableDefaultButton( GetParent( hShellWnd ), TRUE );
		}
		break;

		case WM_PAINT:
		{
			RECT rc;
			PAINTSTRUCT ps;
			BeginPaint( hWnd, &ps );
			HDC hdc = ps.hdc;
			if( hdc )
			{
				HICON hIcon = LoadIcon( g_hInstance, MAKEINTRESOURCE( IDI_LOCKED ) );
				GetClientRect( hWnd, &rc );
				DrawIcon( hdc, ( rc.right / 2 ) - 16 ,rc.bottom / 4, hIcon );
				rc.top = ( rc.bottom / 4 ) + 40;
				SelectObject( hdc, GetStockObject( ANSI_VAR_FONT ) );
				DrawText( hdc, _T("Access to this folder is restricted by IconLock. Please select another location."),-1, &rc, DT_CENTER );
				DestroyIcon( hIcon );
			}
			EndPaint( hWnd, &ps );
			EnableDefaultButton( GetParent(hShellWnd), FALSE );		
		}
		break;
	}
	return DefWindowProc( hWnd, msg, wParam, lParam);
}




/////////////////////////////////////////////////////////////////////////////
//	
void _cdecl DebugOut_CompiledOutOfReleaseBuild( TCHAR* format, ... )
{
	#ifdef _DEBUG

    static TCHAR szBuf[1024];
    static TCHAR sz[30];
	static va_list argptr;
	{
		va_start( argptr, format );
		wvsprintf( szBuf, format, argptr );
		va_end( argptr);
		wsprintf(sz, _T("%X  (LockFolder) "), GetCurrentThreadId() );
		OutputDebugString( sz );
		OutputDebugString( szBuf );
		OutputDebugString( _T("\n\r") );
	}
	#endif

}

